# Copyright 2012 OpenStack Foundation
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools

from tempest.api.compute import base
from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exceptions

CONF = config.CONF


class ImagesTestJSON(base.BaseV2ComputeTest):
    """Test server images"""

    create_default_network = True

    @classmethod
    def skip_checks(cls):
        super(ImagesTestJSON, cls).skip_checks()
        if not CONF.service_available.glance:
            skip_msg = ("%s skipped as glance is not available" % cls.__name__)
            raise cls.skipException(skip_msg)
        if not CONF.compute_feature_enabled.snapshot:
            skip_msg = ("%s skipped as instance snapshotting is not supported"
                        % cls.__name__)
            raise cls.skipException(skip_msg)

    @classmethod
    def setup_clients(cls):
        super(ImagesTestJSON, cls).setup_clients()
        if cls.is_requested_microversion_compatible('2.35'):
            cls.client = cls.compute_images_client
        else:
            cls.client = cls.images_client

    @decorators.idempotent_id('aa06b52b-2db5-4807-b218-9441f75d74e3')
    def test_delete_saving_image(self):
        """Test deleting server image while it is in 'SAVING' state"""
        server = self.create_test_server(wait_until='ACTIVE')
        self.addCleanup(self.servers_client.delete_server, server['id'])
        # wait for server active to avoid conflict when deleting server
        # in task_state image_snapshot
        self.addCleanup(waiters.wait_for_server_status, self.servers_client,
                        server['id'], 'ACTIVE')
        snapshot_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='test-snap')
        try:
            image = self.create_image_from_server(server['id'],
                                                  name=snapshot_name,
                                                  wait_until='SAVING')
            self.client.delete_image(image['id'])
            msg = ('The image with ID {image_id} failed to be deleted'
                   .format(image_id=image['id']))
            self.assertTrue(self.client.is_resource_deleted(image['id']),
                            msg)
            self.assertEqual(snapshot_name, image['name'])
        except lib_exceptions.TimeoutException as ex:
            # If timeout is reached, we don't need to check state,
            # since, it wouldn't be a 'SAVING' state at least and apart from
            # it, this testcase doesn't have scope for other state transition
            # Hence, skip the test.
            raise self.skipException("This test is skipped because " + str(ex))

    @decorators.idempotent_id('aaacd1d0-55a2-4ce8-818a-b5439df8adc9')
    def test_create_image_from_stopped_server(self):
        """Test creating server image from stopped server"""
        server = self.create_test_server(wait_until='ACTIVE')
        self.servers_client.stop_server(server['id'])
        waiters.wait_for_server_status(self.servers_client,
                                       server['id'], 'SHUTOFF')
        self.addCleanup(self.servers_client.delete_server, server['id'])
        snapshot_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='test-snap')
        image = self.create_image_from_server(server['id'],
                                              name=snapshot_name,
                                              wait_until='ACTIVE',
                                              wait_for_server=False)
        # This is required due to ceph issue:
        # https://bugs.launchpad.net/glance/+bug/2045769.
        # New location APIs are async so we need to wait for the location
        # import task to complete.
        # This should work with old location API since we don't fail if there
        # are no tasks for the image
        waiters.wait_for_image_tasks_status(self.images_client,
                                            image['id'], 'success')
        self.addCleanup(self.client.delete_image, image['id'])
        self.assertEqual(snapshot_name, image['name'])

    @decorators.idempotent_id('71bcb732-0261-11e7-9086-fa163e4fa634')
    @testtools.skipUnless(CONF.compute_feature_enabled.pause,
                          'Pause is not available.')
    def test_create_image_from_paused_server(self):
        """Test creating server image from paused server"""
        server = self.create_test_server(wait_until='ACTIVE')
        self.servers_client.pause_server(server['id'])
        waiters.wait_for_server_status(self.servers_client,
                                       server['id'], 'PAUSED')
        self.addCleanup(self.servers_client.delete_server, server['id'])

        snapshot_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='test-snap')
        image = self.create_image_from_server(server['id'],
                                              name=snapshot_name,
                                              wait_until='ACTIVE',
                                              wait_for_server=False)
        # This is required due to ceph issue:
        # https://bugs.launchpad.net/glance/+bug/2045769.
        # New location APIs are async so we need to wait for the location
        # import task to complete.
        # This should work with old location API since we don't fail if there
        # are no tasks for the image
        waiters.wait_for_image_tasks_status(self.images_client,
                                            image['id'], 'success')
        self.addCleanup(self.client.delete_image, image['id'])
        self.assertEqual(snapshot_name, image['name'])

    @decorators.idempotent_id('8ca07fec-0262-11e7-907e-fa163e4fa634')
    @testtools.skipUnless(CONF.compute_feature_enabled.suspend,
                          'Suspend is not available.')
    def test_create_image_from_suspended_server(self):
        """Test creating server image from suspended server"""
        server = self.create_test_server(wait_until='ACTIVE')
        self.servers_client.suspend_server(server['id'])
        waiters.wait_for_server_status(self.servers_client,
                                       server['id'], 'SUSPENDED')
        self.addCleanup(self.servers_client.delete_server, server['id'])

        snapshot_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='test-snap')
        image = self.create_image_from_server(server['id'],
                                              name=snapshot_name,
                                              wait_until='ACTIVE',
                                              wait_for_server=False)
        # This is required due to ceph issue:
        # https://bugs.launchpad.net/glance/+bug/2045769.
        # New location APIs are async so we need to wait for the location
        # import task to complete.
        # This should work with old location API since we don't fail if there
        # are no tasks for the image
        waiters.wait_for_image_tasks_status(self.images_client,
                                            image['id'], 'success')
        self.addCleanup(self.client.delete_image, image['id'])
        self.assertEqual(snapshot_name, image['name'])

    @decorators.idempotent_id('f3cac456-e3fe-4183-a7a7-a59f7f017088')
    def test_create_server_from_snapshot(self):
        # Create one server normally
        server = self.create_test_server(wait_until='ACTIVE')
        self.addCleanup(self.servers_client.delete_server, server['id'])

        # Snapshot it
        snapshot_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='test-snap')
        image = self.create_image_from_server(server['id'],
                                              name=snapshot_name,
                                              wait_until='ACTIVE',
                                              wait_for_server=False)
        self.addCleanup(self.client.delete_image, image['id'])

        # Try to create another server from that snapshot
        server2 = self.create_test_server(wait_until='ACTIVE',
                                          image_id=image['id'])

        # Delete server 2 before we finish otherwise we'll race with
        # the cleanup which tries to delete the image before the
        # server is gone.
        self.servers_client.delete_server(server2['id'])
        waiters.wait_for_server_termination(self.servers_client, server2['id'])



# Copyright 2016 Red Hat.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.lib.services.volume.v3 import messages_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestUserMessagesClient(base.BaseServiceTest):
    USER_MESSAGE_INFO = {
        "created_at": "2016-11-21T06:16:34.000000",
        "guaranteed_until": "2016-12-21T06:16:34.000000",
        "user_message": "No storage could be allocated for this volume "
                        "request. You may be able to try another size or"
                        " volume type.",
        "resource_uuid": "c570b406-bf0b-4067-9398-f0bb09a7d9d7",
        "request_id": "req-8f68681e-9b6b-4009-b94c-ac0811595451",
        "message_level": "ERROR",
        "id": "9a7dafbd-a156-4540-8996-50e71b5dcadf",
        "resource_type": "VOLUME",
        "links": [
            {"href": "http://192.168.100.230:8776/v3/"
                     "a678cb65f701462ea2257245cd640829/messages/"
                     "9a7dafbd-a156-4540-8996-50e71b5dcadf",
             "rel": "self"},
            {"href": "http://192.168.100.230:8776/"
                     "a678cb65f701462ea2257245cd640829/messages/"
                     "9a7dafbd-a156-4540-8996-50e71b5dcadf",
             "rel": "bookmark"}]
        }
    FAKE_SHOW_USER_MESSAGE = {
        "message": dict(event_id="000002", **USER_MESSAGE_INFO)}

    FAKE_LIST_USER_MESSAGES = {
        "messages": [
            dict(event_id="000003", **USER_MESSAGE_INFO),
            dict(event_id="000004", **USER_MESSAGE_INFO)
        ]
    }

    def setUp(self):
        super(TestUserMessagesClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = messages_client.MessagesClient(fake_auth,
                                                     'volume',
                                                     'regionOne')

    def _test_show_user_message(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_message,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_SHOW_USER_MESSAGE,
            bytes_body,
            message_id="9a7dafbd-a156-4540-8996-50e71b5dcadf")

    def _test_list_user_message(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_messages,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_LIST_USER_MESSAGES,
            bytes_body)

    def test_list_user_message_with_str_body(self):
        self._test_list_user_message()

    def test_list_user_message_with_bytes_body(self):
        self._test_list_user_message(bytes_body=True)

    def test_show_user_message_with_str_body(self):
        self._test_show_user_message()

    def test_show_user_message_with_bytes_body(self):
        self._test_show_user_message(bytes_body=True)

    def test_delete_user_message(self):
        self.check_service_client_function(
            self.client.delete_message,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            message_id="9a7dafbd-a156-4540-8996-50e71b5dcadf",
            status=204)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import abc

from oslo_log import log as logging

from tempest.lib import auth
from tempest.lib import exceptions as lib_exc
from tempest.lib.services.identity.v2 import identity_client as v2_identity

LOG = logging.getLogger(__name__)


class CredsClient(object, metaclass=abc.ABCMeta):
    """This class is a wrapper around the identity clients

     to provide a single interface for managing credentials in both v2 and v3
     cases. It's not bound to created credentials, only to a specific set of
     admin credentials used for generating credentials.
    """

    def __init__(self, identity_client, projects_client, users_client,
                 roles_client):
        # The client implies version and credentials
        self.identity_client = identity_client
        self.users_client = users_client
        self.projects_client = projects_client
        self.roles_client = roles_client

    def create_user(self, username, password, project=None, email=None):
        params = {'name': username,
                  'password': password}
        # with keystone v3, a default project is not required
        if project:
            params[self.project_id_param] = project['id']
        # email is not a first-class attribute of a user
        if email:
            params['email'] = email
        user = self.users_client.create_user(**params)
        if 'user' in user:
            user = user['user']
        return user

    def delete_user(self, user_id):
        self.users_client.delete_user(user_id)

    @abc.abstractmethod
    def create_project(self, name, description):
        pass

    @abc.abstractmethod
    def show_project(self, project_id):
        pass

    def _check_role_exists(self, role_name):
        try:
            roles = self._list_roles()
            lc_role_name = role_name.lower()
            role = next(r for r in roles if r['name'].lower() == lc_role_name)
        except StopIteration:
            return None
        return role

    def create_user_role(self, role_name):
        if not self._check_role_exists(role_name):
            self.roles_client.create_role(name=role_name)

    def assign_user_role(self, user, project, role_name):
        role = self._check_role_exists(role_name)
        if not role:
            msg = 'No "%s" role found' % role_name
            raise lib_exc.NotFound(msg)
        try:
            self.roles_client.create_user_role_on_project(project['id'],
                                                          user['id'],
                                                          role['id'])
        except lib_exc.Conflict:
            LOG.debug("Role %s already assigned on project %s for user %s",
                      role['id'], project['id'], user['id'])

    @abc.abstractmethod
    def get_credentials(
            self, user, project, password, domain=None, system=None):
        """Produces a Credentials object from the details provided

        :param user: a user dict
        :param project: a project dict or None if using domain or system scope
        :param password: the password as a string
        :param domain: a domain dict
        :param system: a system dict
        :return: a Credentials object with all the available credential details
        """
        pass

    def _list_roles(self):
        roles = self.roles_client.list_roles()['roles']
        return roles


class V2CredsClient(CredsClient):
    project_id_param = 'tenantId'

    def __init__(self, identity_client, projects_client, users_client,
                 roles_client):
        super(V2CredsClient, self).__init__(identity_client,
                                            projects_client,
                                            users_client,
                                            roles_client)

    def create_project(self, name, description):
        tenant = self.projects_client.create_tenant(
            name=name, description=description)['tenant']
        return tenant

    def show_project(self, project_id):
        return self.projects_client.show_tenant(project_id)['tenant']

    def delete_project(self, project_id):
        self.projects_client.delete_tenant(project_id)

    def get_credentials(
        self, user, project, password, domain=None, system=None):
        # User and project already include both ID and name here,
        # so there's no need to use the fill_in mode
        return auth.get_credentials(
            auth_url=None,
            fill_in=False,
            identity_version='v2',
            username=user['name'], user_id=user['id'],
            tenant_name=project['name'], tenant_id=project['id'],
            password=password)


class V3CredsClient(CredsClient):
    project_id_param = 'project_id'

    def __init__(self, identity_client, projects_client, users_client,
                 roles_client, domains_client, domain_name):
        super(V3CredsClient, self).__init__(identity_client, projects_client,
                                            users_client, roles_client)
        self.domains_client = domains_client

        try:
            # Domain names must be unique, in any case a list is returned,
            # selecting the first (and only) element
            self.creds_domain = self.domains_client.list_domains(
                name=domain_name)['domains'][0]
        except lib_exc.NotFound:
            # TODO(andrea) we could probably create the domain on the fly
            msg = "Requested domain %s could not be found" % domain_name
            raise lib_exc.InvalidCredentials(msg)

    def create_project(self, name, description):
        project = self.projects_client.create_project(
            name=name, description=description,
            domain_id=self.creds_domain['id'])['project']
        return project

    def show_project(self, project_id):
        return self.projects_client.show_project(project_id)['project']

    def delete_project(self, project_id):
        self.projects_client.delete_project(project_id)

    def create_domain(self, name, description):
        domain = self.domains_client.create_domain(
            name=name, description=description)['domain']
        return domain

    def delete_domain(self, domain_id):
        self.domains_client.update_domain(domain_id, enabled=False)
        self.domains_client.delete_domain(domain_id)

    def create_user(self, username, password, project=None, email=None,
                    domain_id=None):
        params = {'name': username,
                  'password': password,
                  'domain_id': domain_id or self.creds_domain['id']}
        # with keystone v3, a default project is not required
        if project:
            params[self.project_id_param] = project['id']
        # email is not a first-class attribute of a user
        if email:
            params['email'] = email
        user = self.users_client.create_user(**params)
        if 'user' in user:
            user = user['user']
        return user

    def get_credentials(
            self, user, project, password, domain=None, system=None):
        # User, project and domain already include both ID and name here,
        # so there's no need to use the fill_in mode.
        # NOTE(andreaf) We need to set all fields in the returned credentials.
        # Scope is then used to pick only those relevant for the type of
        # token needed by each service client.
        if project:
            project_name = project['name']
            project_id = project['id']
        else:
            project_name = None
            project_id = None
        if domain:
            domain_name = domain['name']
            domain_id = domain['id']
        else:
            domain_name = self.creds_domain['name']
            domain_id = self.creds_domain['id']
        return auth.get_credentials(
            auth_url=None,
            fill_in=False,
            identity_version='v3',
            username=user['name'], user_id=user['id'],
            project_name=project_name, project_id=project_id,
            password=password,
            project_domain_id=self.creds_domain['id'],
            project_domain_name=self.creds_domain['name'],
            domain_id=domain_id,
            domain_name=domain_name,
            system=system)

    def assign_user_role_on_domain(self, user, role_name, domain=None):
        """Assign the specified role on a domain

        :param user: a user dict
        :param role_name: name of the role to be assigned
        :param domain: (optional) The domain to assign the role on. If not
                                  specified the default domain of cred_client
        """
        # NOTE(andreaf) This method is very specific to the v3 case, and
        # because of that it's not defined in the parent class.
        if domain is None:
            domain = self.creds_domain
        role = self._check_role_exists(role_name)
        if not role:
            msg = 'No "%s" role found' % role_name
            raise lib_exc.NotFound(msg)
        try:
            self.roles_client.create_user_role_on_domain(
                domain['id'], user['id'], role['id'])
        except lib_exc.Conflict:
            LOG.debug("Role %s already assigned on domain %s for user %s",
                      role['id'], domain['id'], user['id'])

    def assign_user_role_on_system(self, user, role_name):
        """Assign the specified role on the system

        :param user: a user dict
        :param role_name: name of the role to be assigned
        """
        role = self._check_role_exists(role_name)
        if not role:
            msg = 'No "%s" role found' % role_name
            raise lib_exc.NotFound(msg)
        try:
            self.roles_client.create_user_role_on_system(
                user['id'], role['id'])
        except lib_exc.Conflict:
            LOG.debug("Role %s already assigned on the system for user %s",
                      role['id'], user['id'])


def get_creds_client(identity_client,
                     projects_client,
                     users_client,
                     roles_client,
                     domains_client=None,
                     project_domain_name=None):
    if isinstance(identity_client, v2_identity.IdentityClient):
        return V2CredsClient(identity_client, projects_client, users_client,
                             roles_client)
    else:
        return V3CredsClient(identity_client, projects_client, users_client,
                             roles_client, domains_client, project_domain_name)



# Copyright 2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os

import fixtures

from tempest.cmd import init
from tempest.tests import base


class TestTempestInit(base.TestCase):

    def test_generate_stestr_conf(self):
        # Create fake conf dir
        conf_dir = self.useFixture(fixtures.TempDir())

        init_cmd = init.TempestInit(None, None)
        init_cmd.generate_stestr_conf(conf_dir.path)

        # Generate expected file contents
        top_level_path = os.path.dirname(os.path.dirname(init.__file__))
        discover_path = os.path.join(top_level_path, 'test_discover')
        stestr_conf_file = init.STESTR_CONF % (discover_path, top_level_path)

        conf_path = conf_dir.join('.stestr.conf')
        with open(conf_path, 'r') as conf_file:
            self.assertEqual(conf_file.read(), stestr_conf_file)

    def test_generate_sample_config(self):
        local_dir = self.useFixture(fixtures.TempDir())
        etc_dir_path = os.path.join(local_dir.path, 'etc')
        os.mkdir(etc_dir_path)
        init_cmd = init.TempestInit(None, None)
        local_sample_conf_file = os.path.join(etc_dir_path,
                                              'tempest.conf.sample')

        # Verify no sample config file exist
        self.assertFalse(os.path.isfile(local_sample_conf_file))
        init_cmd.generate_sample_config(local_dir.path)

        # Verify sample config file exist with some content
        self.assertTrue(os.path.isfile(local_sample_conf_file))
        self.assertGreater(os.path.getsize(local_sample_conf_file), 0)

    def test_update_local_conf(self):
        local_dir = self.useFixture(fixtures.TempDir())
        etc_dir_path = os.path.join(local_dir.path, 'etc')
        os.mkdir(etc_dir_path)
        lock_dir = os.path.join(local_dir.path, 'tempest_lock')
        config_path = os.path.join(etc_dir_path, 'tempest.conf')
        log_dir = os.path.join(local_dir.path, 'logs')

        init_cmd = init.TempestInit(None, None)

        # Generate the config file
        init_cmd.generate_sample_config(local_dir.path)

        # Create a conf file with populated values
        config_parser_pre = init_cmd.get_configparser(config_path)
        with open(config_path, 'w+') as conf_file:
            # create the same section init will check for and add values to
            config_parser_pre.add_section('oslo_concurrency')
            config_parser_pre.set('oslo_concurrency', 'TEST', local_dir.path)
            # create a new section
            config_parser_pre.add_section('TEST')
            config_parser_pre.set('TEST', 'foo', "bar")
            config_parser_pre.write(conf_file)

        # Update the config file the same way tempest init does
        init_cmd.update_local_conf(config_path, lock_dir, log_dir)

        # parse the new config file to verify it
        config_parser_post = init_cmd.get_configparser(config_path)

        # check that our value in oslo_concurrency wasn't overwritten
        self.assertTrue(config_parser_post.has_section('oslo_concurrency'))
        self.assertEqual(config_parser_post.get('oslo_concurrency', 'TEST'),
                         local_dir.path)
        # check that the lock directory was set correctly
        self.assertEqual(config_parser_post.get('oslo_concurrency',
                                                'lock_path'), lock_dir)

        # check that our new section still exists and wasn't modified
        self.assertTrue(config_parser_post.has_section('TEST'))
        self.assertEqual(config_parser_post.get('TEST', 'foo'), 'bar')

        # check that the DEFAULT values are correct
        # NOTE(auggy): has_section ignores DEFAULT
        self.assertEqual(config_parser_post.get('DEFAULT', 'log_dir'), log_dir)

    def test_create_working_dir_with_existing_local_dir_non_empty(self):
        fake_local_dir = self.useFixture(fixtures.TempDir())
        fake_local_conf_dir = self.useFixture(fixtures.TempDir())
        open("%s/foo" % fake_local_dir.path, 'w').close()

        _init = init.TempestInit(None, None)
        self.assertRaises(OSError,
                          _init.create_working_dir,
                          fake_local_dir.path,
                          fake_local_conf_dir.path)

    def test_create_working_dir(self):
        fake_local_dir = self.useFixture(fixtures.TempDir())
        fake_local_conf_dir = self.useFixture(fixtures.TempDir())
        os.rmdir(fake_local_dir.path)
        # Create a fake conf file
        fake_file = fake_local_conf_dir.join('conf_file.conf')
        open(fake_file, 'w').close()
        init_cmd = init.TempestInit(None, None)
        init_cmd.create_working_dir(fake_local_dir.path,
                                    fake_local_conf_dir.path)
        # Assert directories are created
        lock_path = os.path.join(fake_local_dir.path, 'tempest_lock')
        etc_dir = os.path.join(fake_local_dir.path, 'etc')
        log_dir = os.path.join(fake_local_dir.path, 'logs')
        stestr_dir = os.path.join(fake_local_dir.path, '.stestr')
        self.assertTrue(os.path.isdir(lock_path))
        self.assertTrue(os.path.isdir(etc_dir))
        self.assertTrue(os.path.isdir(log_dir))
        self.assertTrue(os.path.isdir(stestr_dir))
        # Assert file creation
        fake_file_moved = os.path.join(etc_dir, 'conf_file.conf')
        local_conf_file = os.path.join(etc_dir, 'tempest.conf')
        local_stestr_conf = os.path.join(fake_local_dir.path, '.stestr.conf')
        self.assertTrue(os.path.isfile(fake_file_moved))
        self.assertTrue(os.path.isfile(local_conf_file))
        self.assertTrue(os.path.isfile(local_stestr_conf))

    def test_take_action_fails(self):
        class ParsedArgs(object):
            workspace_dir = self.useFixture(fixtures.TempDir()).path
            workspace_path = os.path.join(workspace_dir, 'workspace.yaml')
            name = 'test'
            dir_base = self.useFixture(fixtures.TempDir()).path
            dir = os.path.join(dir_base, 'foo', 'bar')
            config_dir = self.useFixture(fixtures.TempDir()).path
            show_global_dir = False
        pa = ParsedArgs()
        init_cmd = init.TempestInit(None, None)
        self.assertRaises(OSError, init_cmd.take_action, pa)
        # one more trying should be a same error not "workspace already exists"
        self.assertRaises(OSError, init_cmd.take_action, pa)



# Copyright 2017 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from oslo_utils import uuidutils

from tempest.api.compute import base as compute_base
from tempest.common import waiters
from tempest import exceptions
from tempest.lib import exceptions as lib_exc
from tempest.tests import base


class TestBaseV2ComputeTest(base.TestCase):
    """Unit tests for utility functions in BaseV2ComputeTest."""

    @mock.patch.multiple(compute_base.BaseV2ComputeTest,
                         compute_images_client=mock.DEFAULT,
                         images=[], create=True)
    def test_create_image_from_server_no_wait(self, compute_images_client):
        """Tests create_image_from_server without the wait_until kwarg."""
        # setup mocks
        image_id = uuidutils.generate_uuid()
        fake_image = mock.Mock(response={'location': image_id})
        compute_images_client.create_image.return_value = fake_image
        # call the utility method
        cleanup_path = 'tempest.test.BaseTestCase.addClassResourceCleanup'
        with mock.patch(cleanup_path) as mock_cleanup:
            image = compute_base.BaseV2ComputeTest.create_image_from_server(
                mock.sentinel.server_id, name='fake-snapshot-name')
        self.assertEqual(fake_image, image)
        # make our assertions
        compute_images_client.create_image.assert_called_once_with(
            mock.sentinel.server_id, name='fake-snapshot-name')
        mock_cleanup.assert_called_once()
        self.assertIn(image_id, mock_cleanup.call_args[0])

    @mock.patch.multiple(compute_base.BaseV2ComputeTest,
                         compute_images_client=mock.DEFAULT,
                         servers_client=mock.DEFAULT,
                         images=[], create=True)
    @mock.patch.object(waiters, 'wait_for_image_status')
    @mock.patch.object(waiters, 'wait_for_server_status')
    def test_create_image_from_server_wait_until_active(self,
                                                        wait_for_server_status,
                                                        wait_for_image_status,
                                                        servers_client,
                                                        compute_images_client):
        """Tests create_image_from_server with wait_until='ACTIVE' kwarg."""
        # setup mocks
        image_id = uuidutils.generate_uuid()
        fake_image = mock.Mock(response={'location': image_id})
        compute_images_client.create_image.return_value = fake_image
        compute_images_client.show_image.return_value = (
            {'image': fake_image})
        # call the utility method
        image = compute_base.BaseV2ComputeTest.create_image_from_server(
            mock.sentinel.server_id, wait_until='ACTIVE')
        self.assertEqual(fake_image, image)
        # make our assertions
        wait_for_image_status.assert_called_once_with(
            compute_images_client, image_id, 'ACTIVE')
        wait_for_server_status.assert_called_once_with(
            servers_client, mock.sentinel.server_id, 'ACTIVE')
        compute_images_client.show_image.assert_called_once_with(image_id)

    @mock.patch.multiple(compute_base.BaseV2ComputeTest,
                         compute_images_client=mock.DEFAULT,
                         servers_client=mock.DEFAULT,
                         images=[], create=True)
    @mock.patch.object(waiters, 'wait_for_image_status')
    @mock.patch.object(waiters, 'wait_for_server_status')
    def test_create_image_from_server_wait_until_active_no_server_wait(
            self, wait_for_server_status, wait_for_image_status,
            servers_client, compute_images_client):
        """Tests create_image_from_server with wait_until='ACTIVE' kwarg."""
        # setup mocks
        image_id = uuidutils.generate_uuid()
        fake_image = mock.Mock(response={'location': image_id})
        compute_images_client.create_image.return_value = fake_image
        compute_images_client.show_image.return_value = (
            {'image': fake_image})
        # call the utility method
        image = compute_base.BaseV2ComputeTest.create_image_from_server(
            mock.sentinel.server_id, wait_until='ACTIVE',
            wait_for_server=False)
        self.assertEqual(fake_image, image)
        # make our assertions
        wait_for_image_status.assert_called_once_with(
            compute_images_client, image_id, 'ACTIVE')
        self.assertEqual(0, wait_for_server_status.call_count)
        compute_images_client.show_image.assert_called_once_with(image_id)

    @mock.patch.multiple(compute_base.BaseV2ComputeTest,
                         compute_images_client=mock.DEFAULT,
                         servers_client=mock.DEFAULT,
                         images=[], create=True)
    @mock.patch.object(waiters, 'wait_for_image_status',
                       side_effect=lib_exc.NotFound)
    def _test_create_image_from_server_wait_until_active_not_found(
            self, wait_for_image_status, compute_images_client,
            servers_client, fault=None):
        # setup mocks
        image_id = uuidutils.generate_uuid()
        fake_image = mock.Mock(response={'location': image_id})
        compute_images_client.create_image.return_value = fake_image
        fake_server = {'id': mock.sentinel.server_id}
        if fault:
            fake_server['fault'] = fault
        servers_client.show_server.return_value = {'server': fake_server}
        # call the utility method
        ex = self.assertRaises(
            exceptions.SnapshotNotFoundException,
            compute_base.BaseV2ComputeTest.create_image_from_server,
            mock.sentinel.server_id, wait_until='active')
        # make our assertions
        if fault:
            self.assertIn(fault, str(ex))
        else:
            self.assertNotIn(fault, str(ex))
        if compute_base.BaseV2ComputeTest.is_requested_microversion_compatible(
            '2.35'):
            status = 'ACTIVE'
        else:
            status = 'active'
        wait_for_image_status.assert_called_once_with(
            compute_images_client, image_id, status)
        servers_client.show_server.assert_called_once_with(
            mock.sentinel.server_id)

    def test_create_image_from_server_wait_until_active_not_found_no_fault(
            self):
        # Tests create_image_from_server with wait_until='active' kwarg and
        # the a 404 is raised while waiting for the image status to change. In
        # this test the server does not have a fault associated with it.
        self._test_create_image_from_server_wait_until_active_not_found()

    def test_create_image_from_server_wait_until_active_not_found_with_fault(
            self):
        # Tests create_image_from_server with wait_until='active' kwarg and
        # the a 404 is raised while waiting for the image status to change. In
        # this test the server has a fault associated with it.
        self._test_create_image_from_server_wait_until_active_not_found(
            fault='Lost connection to hypervisor!')

    @mock.patch.multiple(compute_base.BaseV2ComputeTest,
                         compute_images_client=mock.DEFAULT,
                         images=[], create=True)
    @mock.patch.object(waiters, 'wait_for_image_status',
                       side_effect=lib_exc.NotFound)
    def test_create_image_from_server_wait_until_saving_not_found(
            self, wait_for_image_status, compute_images_client):
        # Tests create_image_from_server with wait_until='SAVING' kwarg and
        # the a 404 is raised while waiting for the image status to change. In
        # this case we do not get the server details and just re-raise the 404.
        # setup mocks
        image_id = uuidutils.generate_uuid()
        fake_image = mock.Mock(response={'location': image_id})
        compute_images_client.create_image.return_value = fake_image
        # call the utility method
        self.assertRaises(
            lib_exc.NotFound,
            compute_base.BaseV2ComputeTest.create_image_from_server,
            mock.sentinel.server_id, wait_until='SAVING')
        # make our assertions
        wait_for_image_status.assert_called_once_with(
            compute_images_client, image_id, 'SAVING')

    def _test_version_compatible(self, max_version, expected=True):
        actual = (compute_base.BaseV2ComputeTest.
                  is_requested_microversion_compatible(max_version))
        self.assertEqual(expected, actual)

    def test_check_lower_version(self):
        compute_base.BaseV2ComputeTest.request_microversion = '2.8'
        self._test_version_compatible('2.40')

    def test_check_euqal_version(self):
        compute_base.BaseV2ComputeTest.request_microversion = '2.40'
        self._test_version_compatible('2.40')

    def test_check_higher_version(self):
        compute_base.BaseV2ComputeTest.request_microversion = '2.41'
        self._test_version_compatible('2.40', expected=False)



# Copyright 2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from urllib import parse as urllib

from oslo_serialization import jsonutils as json

from tempest.lib.common import rest_client


class TrustsClient(rest_client.RestClient):
    api_version = "v3"

    def create_trust(self, **kwargs):
        """Creates a trust.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v3-ext/index.html#create-trust
        """
        post_body = json.dumps({'trust': kwargs})
        resp, body = self.post('OS-TRUST/trusts', post_body)
        self.expected_success(201, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def delete_trust(self, trust_id):
        """Deletes a trust."""
        resp, body = self.delete("OS-TRUST/trusts/%s" % trust_id)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def list_trusts(self, **params):
        """Returns trusts

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v3-ext/index.html#list-trusts
        """
        url = "OS-TRUST/trusts/"
        if params:
            url += '?%s' % urllib.urlencode(params)
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def show_trust(self, trust_id):
        """GET trust."""
        resp, body = self.get("OS-TRUST/trusts/%s" % trust_id)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def list_trust_roles(self, trust_id):
        """GET roles delegated by a trust."""
        resp, body = self.get("OS-TRUST/trusts/%s/roles" % trust_id)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def show_trust_role(self, trust_id, role_id):
        """GET role delegated by a trust."""
        resp, body = self.get("OS-TRUST/trusts/%s/roles/%s"
                              % (trust_id, role_id))
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def check_trust_role(self, trust_id, role_id):
        """HEAD Check if role is delegated by a trust."""
        resp, body = self.head("OS-TRUST/trusts/%s/roles/%s"
                               % (trust_id, role_id))
        self.expected_success(200, resp.status)
        return rest_client.ResponseBody(resp, body)



# Copyright 2013 IBM Corp.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import testtools

from tempest.api.identity import base
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

CONF = config.CONF


class GroupsV3TestJSON(base.BaseIdentityV3AdminTest):
    """Test keystone groups"""

    # NOTE: force_tenant_isolation is true in the base class by default but
    # overridden to false here to allow test execution for clouds using the
    # pre-provisioned credentials provider.
    force_tenant_isolation = False

    @classmethod
    def resource_setup(cls):
        super(GroupsV3TestJSON, cls).resource_setup()
        cls.domain = cls.create_domain()

    @decorators.idempotent_id('2e80343b-6c81-4ac3-88c7-452f3e9d5129')
    def test_group_create_update_get(self):
        """Test creating, updating and getting keystone group"""
        prefix = CONF.resource_name_prefix
        # Verify group creation works.
        name = data_utils.rand_name(name='Group', prefix=prefix)
        description = data_utils.rand_name(name='Description', prefix=prefix)
        group = self.setup_test_group(name=name, domain_id=self.domain['id'],
                                      description=description)
        self.assertEqual(group['name'], name)
        self.assertEqual(group['description'], description)
        self.assertEqual(self.domain['id'], group['domain_id'])

        # Verify updating name and description works.
        first_name_update = data_utils.rand_name(
            name='UpdateGroup', prefix=prefix)
        first_desc_update = data_utils.rand_name(
            name='UpdateDescription', prefix=prefix)
        updated_group = self.groups_client.update_group(
            group['id'], name=first_name_update,
            description=first_desc_update)['group']
        self.assertEqual(updated_group['name'], first_name_update)
        self.assertEqual(updated_group['description'], first_desc_update)

        # Verify that the updated values are reflected after performing show.
        new_group = self.groups_client.show_group(group['id'])['group']
        self.assertEqual(group['id'], new_group['id'])
        self.assertEqual(first_name_update, new_group['name'])
        self.assertEqual(first_desc_update, new_group['description'])

        # Verify that updating a single field for a group (name) leaves the
        # other fields (description, domain_id) unchanged.
        second_name_update = data_utils.rand_name(
            self.__class__.__name__ + 'UpdateGroup', prefix=prefix)
        updated_group = self.groups_client.update_group(
            group['id'], name=second_name_update)['group']
        self.assertEqual(second_name_update, updated_group['name'])
        # Verify that 'description' and 'domain_id' were not updated or
        # deleted.
        self.assertEqual(first_desc_update, updated_group['description'])
        self.assertEqual(self.domain['id'], updated_group['domain_id'])

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('1598521a-2f36-4606-8df9-30772bd51339')
    @testtools.skipIf(CONF.identity_feature_enabled.immutable_user_source,
                      'Skipped because environment has an '
                      'immutable user source and solely '
                      'provides read-only access to users.')
    def test_group_users_add_list_delete(self):
        """Test adding/listing/deleting group users"""
        group = self.setup_test_group(domain_id=self.domain['id'])
        # add user into group
        users = []
        for _ in range(3):
            user = self.create_test_user()
            users.append(user)
            self.groups_client.add_group_user(group['id'], user['id'])

        # list users in group
        group_users = self.groups_client.list_group_users(group['id'])['users']
        self.assertEqual(sorted(users, key=lambda k: k['name']),
                         sorted(group_users, key=lambda k: k['name']))
        # check and delete user in group
        for user in users:
            self.groups_client.check_group_user_existence(
                group['id'], user['id'])
            self.groups_client.delete_group_user(group['id'], user['id'])
        group_users = self.groups_client.list_group_users(group['id'])['users']
        self.assertEqual(len(group_users), 0)

    @decorators.idempotent_id('64573281-d26a-4a52-b899-503cb0f4e4ec')
    @testtools.skipIf(CONF.identity_feature_enabled.immutable_user_source,
                      'Skipped because environment has an '
                      'immutable user source and solely '
                      'provides read-only access to users.')
    def test_list_user_groups(self):
        """Test listing user groups when the user is in two groups"""
        # create a user
        user = self.create_test_user()
        # create two groups, and add user into them
        groups = []
        for _ in range(2):
            group = self.setup_test_group(domain_id=self.domain['id'])
            groups.append(group)
            self.groups_client.add_group_user(group['id'], user['id'])
        # list groups which user belongs to
        user_groups = self.users_client.list_user_groups(user['id'])['groups']
        # The `membership_expires_at` attribute is present when listing user
        # group memberships, and is not an attribute of the groups themselves.
        # Therefore we remove it from the comparison.
        for g in user_groups:
            if 'membership_expires_at' in g:
                self.assertIsNone(g['membership_expires_at'])
                del g['membership_expires_at']
        self.assertEqual(sorted(groups, key=lambda k: k['name']),
                         sorted(user_groups, key=lambda k: k['name']))
        self.assertEqual(2, len(user_groups))

    @decorators.idempotent_id('cc9a57a5-a9ed-4f2d-a29f-4f979a06ec71')
    def test_list_groups(self):
        """Test listing groups"""
        group_ids = list()
        fetched_ids = list()
        for _ in range(3):
            group = self.setup_test_group(domain_id=self.domain['id'])
            group_ids.append(group['id'])
        # List and Verify Groups
        # When domain specific drivers are enabled the operations
        # of listing all users and listing all groups are not supported,
        # they need a domain filter to be specified
        if CONF.identity_feature_enabled.domain_specific_drivers:
            body = self.groups_client.list_groups(
                domain_id=self.domain['id'])['groups']
        else:
            body = self.groups_client.list_groups()['groups']
        for g in body:
            fetched_ids.append(g['id'])
        missing_groups = [g for g in group_ids if g not in fetched_ids]
        self.assertEmpty(missing_groups)



# Copyright 2014 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

add_remove_list_flavor_access = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'flavor_access': {
                'type': 'array',
                'items': {
                    'type': 'object',
                    'properties': {
                        'flavor_id': {'type': 'string'},
                        'tenant_id': {'type': 'string'},
                    },
                    'additionalProperties': False,
                    'required': ['flavor_id', 'tenant_id'],
                }
            }
        },
        'additionalProperties': False,
        'required': ['flavor_access']
    }
}



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_serialization import jsonutils as json

from tempest.lib.common import rest_client
from tempest.lib.services.network import base


class TagsClient(base.BaseNetworkClient):

    def create_tag(self, resource_type, resource_id, tag):
        """Adds a tag on the resource.

        For more information, please refer to the official API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#add-a-tag
        """
        uri = '/%s/%s/tags/%s' % (resource_type, resource_id, tag)
        return self.update_resource(
            uri, json.dumps({}), expect_response_code=201,
            expect_empty_body=True)

    def check_tag_existence(self, resource_type, resource_id, tag):
        """Confirm that a given tag is set on the resource.

        For more information, please refer to the official API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#confirm-a-tag
        """
        # TODO(felipemonteiro): Use the "check_resource" method in
        # ``BaseNetworkClient`` once it has been implemented.
        uri = '%s/%s/%s/tags/%s' % (
            self.uri_prefix, resource_type, resource_id, tag)
        resp, _ = self.get(uri)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp)

    def update_all_tags(self, resource_type, resource_id, tags):
        """Replace all tags on the resource.

        For more information, please refer to the official API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#replace-all-tags
        """
        uri = '/%s/%s/tags' % (resource_type, resource_id)
        put_body = {"tags": tags}
        return self.update_resource(uri, put_body)

    def delete_tag(self, resource_type, resource_id, tag):
        """Removes a tag on the resource.

        For more information, please refer to the official API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#remove-a-tag
        """
        uri = '/%s/%s/tags/%s' % (resource_type, resource_id, tag)
        return self.delete_resource(uri)

    def delete_all_tags(self, resource_type, resource_id):
        """Removes all tags on the resource.

        For more information, please refer to the official API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#remove-all-tags
        """
        uri = '/%s/%s/tags' % (resource_type, resource_id)
        return self.delete_resource(uri)

    def list_tags(self, resource_type, resource_id):
        """Retrieves the tags for a resource.

        For more information, please refer to the official API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#obtain-tag-list
        """
        uri = '/%s/%s/tags' % (resource_type, resource_id)
        return self.list_resources(uri)



# Copyright 2012 NTT Data
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_serialization import jsonutils as json

from tempest.lib.api_schema.response.compute.v2_1\
    import quota_classes as schema
from tempest.lib.api_schema.response.compute.v2_50 import quota_classes \
    as schemav250
from tempest.lib.api_schema.response.compute.v2_57 import quota_classes \
    as schemav257
from tempest.lib.common import rest_client
from tempest.lib.services.compute import base_compute_client


class QuotaClassesClient(base_compute_client.BaseComputeClient):

    schema_versions_info = [
        {'min': None, 'max': '2.49', 'schema': schema},
        {'min': '2.50', 'max': '2.56', 'schema': schemav250},
        {'min': '2.57', 'max': None, 'schema': schemav257}]

    def show_quota_class_set(self, quota_class_id):
        """List the quota class set for a quota class."""

        url = 'os-quota-class-sets/%s' % quota_class_id
        resp, body = self.get(url)
        body = json.loads(body)
        _schema = self.get_schema(self.schema_versions_info)
        self.validate_response(_schema.get_quota_class_set, resp, body)
        return rest_client.ResponseBody(resp, body)

    def update_quota_class_set(self, quota_class_id, **kwargs):
        """Update the quota class's limits for one or more resources.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#create-or-update-quotas-for-quota-class
        """
        post_body = json.dumps({'quota_class_set': kwargs})

        resp, body = self.put('os-quota-class-sets/%s' % quota_class_id,
                              post_body)

        body = json.loads(body)
        _schema = self.get_schema(self.schema_versions_info)
        self.validate_response(_schema.update_quota_class_set,
                               resp, body)
        return rest_client.ResponseBody(resp, body)



# Copyright 2013 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy


class fake_httplib2(object):

    def __init__(self, return_type=None, *args, **kwargs):
        self.return_type = return_type

    def request(self, uri, method="GET", body=None, headers=None,
                redirections=5, connection_type=None, chunked=False,
                preload_content=False):
        if not self.return_type:
            fake_headers = fake_http_response(headers)
            return_obj = {
                'uri': uri,
                'method': method,
                'body': body,
                'headers': headers,
                'chunked': chunked,
                'preload_content': preload_content,
            }
            return (fake_headers, return_obj)
        elif isinstance(self.return_type, int):
            body = body or "fake_body"
            header_info = {
                'content-type': 'text/plain',
                'content-length': len(body)
            }
            resp_header = fake_http_response(header_info,
                                             status=self.return_type)
            return (resp_header, body)
        else:
            msg = "unsupported return type %s" % self.return_type
            raise TypeError(msg)


class fake_http_response(dict):
    def __init__(self, headers, body=None,
                 version=1.0, status=200, reason="Ok"):
        """Initialization of fake HTTP Response

        :param headers: dict representing HTTP response headers
        :param body: file-like object
        :param version: HTTP Version
        :param status: Response status code
        :param reason: Status code related message.
        """
        self.body = body
        self.status = status
        self['status'] = str(self.status)
        self.reason = reason
        self.version = version
        self.headers = headers

        if headers:
            for key, value in headers.items():
                self[key.lower()] = value

    def getheaders(self):
        return copy.deepcopy(self.headers).items()

    def getheader(self, key, default):
        return self.headers.get(key, default)

    def read(self, amt):
        return self.body.read(amt)



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os

from oslo_concurrency import lockutils

from tempest import config
from tempest.lib import auth
from tempest.lib.common.rest_client import RestClient
from tempest.lib import exceptions as lib_exc
from tempest.lib.services import clients

CONF = config.CONF


class Manager(clients.ServiceClients):
    """Top level manager for OpenStack tempest clients"""

    def __init__(self, credentials, scope='project'):
        """Initialization of Manager class.

        Setup all services clients and make them available for tests cases.
        :param credentials: type Credentials or TestResources
        :param scope: default scope for tokens produced by the auth provider
        """
        _, identity_uri = get_auth_provider_class(credentials)
        super(Manager, self).__init__(
            credentials=credentials, identity_uri=identity_uri, scope=scope,
            region=CONF.identity.region)
        if CONF.record_resources:
            RestClient.lock_dir = os.path.join(
                lockutils.get_lock_path(CONF),
                'tempest-rec-rw-lock')
            RestClient.record_resources = True
        # TODO(andreaf) When clients are initialised without the right
        # parameters available, the calls below will trigger a KeyError.
        # We should catch that and raise a better error.
        self._set_compute_clients()
        self._set_identity_clients()
        self._set_volume_clients()
        self._set_object_storage_clients()
        self._set_image_clients()
        self._set_network_clients()
        self._set_placement_clients()
        # TODO(andreaf) This is maintained for backward compatibility
        # with plugins, but it should removed eventually, since it was
        # never a stable interface and it's not useful anyways
        self.default_params = config.service_client_config()

    def _set_network_clients(self):
        self.network_agents_client = self.network.AgentsClient()
        self.network_extensions_client = self.network.ExtensionsClient()
        self.networks_client = self.network.NetworksClient()
        self.subnetpools_client = self.network.SubnetpoolsClient()
        self.subnets_client = self.network.SubnetsClient()
        self.ports_client = self.network.PortsClient()
        self.network_quotas_client = self.network.QuotasClient()
        self.floating_ips_client = self.network.FloatingIPsClient()
        self.floating_ips_port_forwarding_client =\
            self.network.FloatingIpsPortForwardingClient()
        self.metering_labels_client = self.network.MeteringLabelsClient()
        self.metering_label_rules_client = (
            self.network.MeteringLabelRulesClient())
        self.routers_client = self.network.RoutersClient()
        self.security_group_rules_client = (
            self.network.SecurityGroupRulesClient())
        self.security_groups_client = self.network.SecurityGroupsClient()
        self.network_versions_client = self.network.NetworkVersionsClient()
        self.service_providers_client = self.network.ServiceProvidersClient()
        self.tags_client = self.network.TagsClient()
        self.qos_client = self.network.QosClient()
        self.qos_min_bw_client = self.network.QosMinimumBandwidthRulesClient()
        self.qos_limit_bw_client = self.network.QosLimitBandwidthRulesClient()
        self.qos_min_pps_client = (
            self.network.QosMinimumPacketRateRulesClient())
        self.segments_client = self.network.SegmentsClient()
        self.trunks_client = self.network.TrunksClient()
        self.log_resource_client = self.network.LogResourceClient()
        self.loggable_resource_client = self.network.LoggableResourceClient()

    def _set_image_clients(self):
        if CONF.service_available.glance:
            self.image_client_v2 = self.image_v2.ImagesClient()
            self.image_member_client_v2 = self.image_v2.ImageMembersClient()
            self.image_cache_client = self.image_v2.ImageCacheClient()
            self.namespaces_client = self.image_v2.NamespacesClient()
            self.resource_types_client = self.image_v2.ResourceTypesClient()
            self.namespace_objects_client = \
                self.image_v2.NamespaceObjectsClient()
            self.schemas_client = self.image_v2.SchemasClient()
            self.namespace_properties_client = \
                self.image_v2.NamespacePropertiesClient()
            self.namespace_tags_client = self.image_v2.NamespaceTagsClient()
            self.image_versions_client = self.image_v2.VersionsClient()
            self.tasks_client = self.image_v2.TaskClient()
            # NOTE(danms): If no alternate endpoint is configured,
            # this client will work the same as the base self.images_client.
            # If your test needs to know if these are different, check the
            # config option to see if the alternate_image_endpoint is set.
            self.image_client_remote = self.image_v2.ImagesClient(
                service=CONF.image.alternate_image_endpoint,
                endpoint_type=CONF.image.alternate_image_endpoint_type,
                region=CONF.image.region)
            # NOTE(abhishekk): If no alternate endpoint is configured,
            # this client will work the same as the base
            # self.image_cache_client. If your test needs to know if
            # these are different, check the config option to see if
            # the alternate_image_endpoint is set.
            self.cache_client_remote = self.image_v2.ImageCacheClient(
                service=CONF.image.alternate_image_endpoint,
                endpoint_type=CONF.image.alternate_image_endpoint_type,
                region=CONF.image.region)

    def _set_compute_clients(self):
        self.agents_client = self.compute.AgentsClient()
        self.compute_networks_client = self.compute.NetworksClient()
        self.migrations_client = self.compute.MigrationsClient()
        self.security_group_default_rules_client = (
            self.compute.SecurityGroupDefaultRulesClient())
        self.certificates_client = self.compute.CertificatesClient()
        eip = CONF.compute_feature_enabled.enable_instance_password
        self.servers_client = self.compute.ServersClient(
            enable_instance_password=eip)
        self.server_groups_client = self.compute.ServerGroupsClient()
        self.limits_client = self.compute.LimitsClient()
        self.keypairs_client = self.compute.KeyPairsClient(
            ssh_key_type=CONF.validation.ssh_key_type)
        self.quotas_client = self.compute.QuotasClient()
        self.quota_classes_client = self.compute.QuotaClassesClient()
        self.flavors_client = self.compute.FlavorsClient()
        self.extensions_client = self.compute.ExtensionsClient()
        self.compute_floating_ips_client = self.compute.FloatingIPsClient()
        self.compute_security_group_rules_client = (
            self.compute.SecurityGroupRulesClient())
        self.compute_security_groups_client = (
            self.compute.SecurityGroupsClient())
        self.interfaces_client = self.compute.InterfacesClient()
        self.availability_zone_client = self.compute.AvailabilityZoneClient()
        self.aggregates_client = self.compute.AggregatesClient()
        self.services_client = self.compute.ServicesClient()
        self.tenant_usages_client = self.compute.TenantUsagesClient()
        self.hosts_client = self.compute.HostsClient()
        self.hypervisor_client = self.compute.HypervisorClient()
        self.instance_usages_audit_log_client = (
            self.compute.InstanceUsagesAuditLogClient())
        self.tenant_networks_client = self.compute.TenantNetworksClient()
        self.assisted_volume_snapshots_client = (
            self.compute.AssistedVolumeSnapshotsClient())
        self.server_external_events_client = (
            self.compute.ServerExternalEventsClient())

        # NOTE: The following client needs special timeout values because
        # the API is a proxy for the other component.
        params_volume = {
            'build_interval': CONF.volume.build_interval,
            'build_timeout': CONF.volume.build_timeout
        }
        self.volumes_extensions_client = self.compute.VolumesClient(
            **params_volume)
        self.compute_versions_client = self.compute.VersionsClient(
            **params_volume)
        self.snapshots_extensions_client = self.compute.SnapshotsClient(
            **params_volume)
        self.compute_images_client = self.compute.ImagesClient(
            build_timeout=CONF.image.build_timeout)

    def _set_placement_clients(self):
        self.placement_client = self.placement.PlacementClient()
        self.resource_providers_client = \
            self.placement.ResourceProvidersClient()

    def _set_identity_clients(self):
        # Clients below use the endpoint type of Keystone API v3, which is set
        # in endpoint_type
        params_v3 = {'endpoint_type': CONF.identity.v3_endpoint_type}
        self.domains_client = self.identity_v3.DomainsClient(**params_v3)
        self.identity_v3_client = self.identity_v3.IdentityClient(**params_v3)
        self.trusts_client = self.identity_v3.TrustsClient(**params_v3)
        self.users_v3_client = self.identity_v3.UsersClient(**params_v3)
        self.endpoints_v3_client = self.identity_v3.EndPointsClient(
            **params_v3)
        self.roles_v3_client = self.identity_v3.RolesClient(**params_v3)
        self.inherited_roles_client = self.identity_v3.InheritedRolesClient(
            **params_v3)
        self.role_assignments_client = self.identity_v3.RoleAssignmentsClient(
            **params_v3)
        self.identity_services_v3_client = self.identity_v3.ServicesClient(
            **params_v3)
        self.policies_client = self.identity_v3.PoliciesClient(**params_v3)
        self.projects_client = self.identity_v3.ProjectsClient(**params_v3)
        self.regions_client = self.identity_v3.RegionsClient(**params_v3)
        self.credentials_client = self.identity_v3.CredentialsClient(
            **params_v3)
        self.groups_client = self.identity_v3.GroupsClient(**params_v3)
        self.identity_versions_v3_client = self.identity_v3.VersionsClient(
            **params_v3)
        self.oauth_consumers_client = self.identity_v3.OAUTHConsumerClient(
            **params_v3)
        self.oauth_token_client = self.identity_v3.OAUTHTokenClient(
            **params_v3)
        self.domain_config_client = self.identity_v3.DomainConfigurationClient(
            **params_v3)
        self.endpoint_filter_client = \
            self.identity_v3.EndPointsFilterClient(**params_v3)
        self.endpoint_groups_client = self.identity_v3.EndPointGroupsClient(
            **params_v3)
        self.catalog_client = self.identity_v3.CatalogClient(**params_v3)
        self.project_tags_client = self.identity_v3.ProjectTagsClient(
            **params_v3)
        self.application_credentials_client = \
            self.identity_v3.ApplicationCredentialsClient(**params_v3)
        self.access_rules_client = \
            self.identity_v3.AccessRulesClient(**params_v3)
        self.identity_limits_client = \
            self.identity_v3.LimitsClient(**params_v3)

        if CONF.identity_feature_enabled.api_v3:
            if CONF.identity.uri_v3:
                self.token_v3_client = self.identity_v3.V3TokenClient(
                    auth_url=CONF.identity.uri_v3)
            else:
                msg = 'Identity v3 API enabled, but no identity.uri_v3 set'
                raise lib_exc.InvalidConfiguration(msg)

    def _set_volume_clients(self):

        self.backups_client_latest = self.volume_v3.BackupsClient()
        self.encryption_types_client_latest = \
            self.volume_v3.EncryptionTypesClient()
        self.snapshot_manage_client_latest = \
            self.volume_v3.SnapshotManageClient()
        self.snapshots_client_latest = self.volume_v3.SnapshotsClient()
        self.volume_capabilities_client_latest = \
            self.volume_v3.CapabilitiesClient()
        self.volume_manage_client_latest = (
            self.volume_v3.VolumeManageClient())
        self.volume_qos_client_latest = self.volume_v3.QosSpecsClient()
        self.volume_services_client_latest = (
            self.volume_v3.ServicesClient())
        self.volume_types_client_latest = self.volume_v3.TypesClient()
        self.volume_hosts_client_latest = self.volume_v3.HostsClient()
        self.volume_quotas_client_latest = self.volume_v3.QuotasClient()
        self.volume_quota_classes_client_latest = \
            self.volume_v3.QuotaClassesClient()
        self.volume_scheduler_stats_client_latest = \
            self.volume_v3.SchedulerStatsClient()
        self.volume_transfers_client_latest = \
            self.volume_v3.TransfersClient()
        self.volume_transfers_mv355_client_latest = \
            self.volume_v3.TransfersV355Client()
        self.volume_availability_zone_client_latest = \
            self.volume_v3.AvailabilityZoneClient()
        self.volume_limits_client_latest = self.volume_v3.LimitsClient()
        self.volumes_client_latest = self.volume_v3.VolumesClient()
        self.volumes_extension_client_latest = \
            self.volume_v3.ExtensionsClient()
        self.group_types_client_latest = self.volume_v3.GroupTypesClient()
        self.groups_client_latest = self.volume_v3.GroupsClient()
        self.group_snapshots_client_latest = \
            self.volume_v3.GroupSnapshotsClient()
        self.volume_messages_client_latest = (
            self.volume_v3.MessagesClient())
        self.volume_versions_client_latest = (
            self.volume_v3.VersionsClient())
        self.attachments_client_latest = (
            self.volume_v3.AttachmentsClient())

        # TODO(gmann): Below alias for service clients have been
        # deprecated and will be removed in future. Start using the alias
        # defined above with suffix _latest.
        # ****************Deprecated alias start from here***************
        self.backups_v2_client = self.volume_v3.BackupsClient()
        self.encryption_types_v2_client = \
            self.volume_v3.EncryptionTypesClient()
        self.snapshot_manage_v2_client = \
            self.volume_v3.SnapshotManageClient()
        self.snapshots_v2_client = self.volume_v3.SnapshotsClient()
        self.volume_capabilities_v2_client = \
            self.volume_v3.CapabilitiesClient()
        self.volume_manage_v2_client = self.volume_v3.VolumeManageClient()
        self.volume_qos_v2_client = self.volume_v3.QosSpecsClient()
        self.volume_services_v2_client = self.volume_v3.ServicesClient()
        self.volume_types_v2_client = self.volume_v3.TypesClient()
        self.volume_hosts_v2_client = self.volume_v3.HostsClient()
        self.volume_quotas_v2_client = self.volume_v3.QuotasClient()
        self.volume_quota_classes_v2_client = \
            self.volume_v3.QuotaClassesClient()
        self.volume_scheduler_stats_v2_client = \
            self.volume_v3.SchedulerStatsClient()
        self.volume_transfers_v2_client = self.volume_v3.TransfersClient()
        self.volume_v2_availability_zone_client = \
            self.volume_v3.AvailabilityZoneClient()
        self.volume_v2_limits_client = self.volume_v3.LimitsClient()
        self.volumes_v2_client = self.volume_v3.VolumesClient()
        self.volumes_v2_extension_client = \
            self.volume_v3.ExtensionsClient()

        self.backups_v3_client = self.volume_v3.BackupsClient()
        self.group_types_v3_client = self.volume_v3.GroupTypesClient()
        self.groups_v3_client = self.volume_v3.GroupsClient()
        self.group_snapshots_v3_client = \
            self.volume_v3.GroupSnapshotsClient()
        self.snapshots_v3_client = self.volume_v3.SnapshotsClient()
        self.volume_v3_messages_client = self.volume_v3.MessagesClient()
        self.volume_v3_versions_client = self.volume_v3.VersionsClient()
        self.volumes_v3_client = self.volume_v3.VolumesClient()
        # ****************Deprecated alias end here***********************

    def _set_object_storage_clients(self):
        self.account_client = self.object_storage.AccountClient()
        self.bulk_client = self.object_storage.BulkMiddlewareClient()
        self.capabilities_client = self.object_storage.CapabilitiesClient()
        self.container_client = self.object_storage.ContainerClient()
        self.object_client = self.object_storage.ObjectClient()


def get_auth_provider_class(credentials):
    if isinstance(credentials, auth.KeystoneV3Credentials):
        return auth.KeystoneV3AuthProvider, CONF.identity.uri_v3
    else:
        return auth.KeystoneV2AuthProvider, CONF.identity.uri


def get_auth_provider(credentials, pre_auth=False, scope='project'):
    # kwargs for auth provider match the common ones used by service clients
    default_params = config.service_client_config()
    if credentials is None:
        raise lib_exc.InvalidCredentials(
            'Credentials must be specified')
    auth_provider_class, auth_url = get_auth_provider_class(
        credentials)
    _auth_provider = auth_provider_class(credentials, auth_url,
                                         scope=scope,
                                         **default_params)
    if pre_auth:
        _auth_provider.set_auth()
    return _auth_provider



# Copyright 2014 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy


list_hosts = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'hosts': {
                'type': 'array',
                'items': {
                    'type': 'object',
                    'properties': {
                        'host_name': {'type': 'string'},
                        'service': {'type': 'string'},
                        'zone': {'type': 'string'}
                    },
                    'additionalProperties': False,
                    'required': ['host_name', 'service', 'zone']
                }
            }
        },
        'additionalProperties': False,
        'required': ['hosts']
    }
}

get_host_detail = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'host': {
                'type': 'array',
                'item': {
                    'type': 'object',
                    'properties': {
                        'resource': {
                            'type': 'object',
                            'properties': {
                                'cpu': {'type': 'integer'},
                                'disk_gb': {'type': 'integer'},
                                'host': {'type': 'string'},
                                'memory_mb': {'type': 'integer'},
                                'project': {'type': 'string'}
                            },
                            'additionalProperties': False,
                            'required': ['cpu', 'disk_gb', 'host',
                                         'memory_mb', 'project']
                        }
                    },
                    'additionalProperties': False,
                    'required': ['resource']
                }
            }
        },
        'additionalProperties': False,
        'required': ['host']
    }
}

startup_host = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'host': {'type': 'string'},
            'power_action': {'enum': ['startup']}
        },
        'additionalProperties': False,
        'required': ['host', 'power_action']
    }
}

# The 'power_action' attribute of 'shutdown_host' API is 'shutdown'
shutdown_host = copy.deepcopy(startup_host)

shutdown_host['response_body']['properties']['power_action'] = {
    'enum': ['shutdown']
}

# The 'power_action' attribute of 'reboot_host' API is 'reboot'
reboot_host = copy.deepcopy(startup_host)

reboot_host['response_body']['properties']['power_action'] = {
    'enum': ['reboot']
}

update_host = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'host': {'type': 'string'},
            'maintenance_mode': {'enum': ['on_maintenance',
                                          'off_maintenance']},
            'status': {'enum': ['enabled', 'disabled']}
        },
        'additionalProperties': False,
        'anyOf': [
            {'required': ['host', 'status']},
            {'required': ['host', 'maintenance_mode']}
        ]
    }
}



# Copyright 2021 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from tempest.lib.services.identity.v3 import limits_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestLimitsClient(base.BaseServiceTest):
    def setUp(self):
        super(TestLimitsClient, self).setUp()
        self.client = limits_client.LimitsClient(
            fake_auth_provider.FakeAuthProvider(),
            'identity', 'regionOne')

    def test_get_registered_limits(self):
        fake_result = {'foo': 'bar'}
        self.check_service_client_function(
            self.client.get_registered_limits,
            'tempest.lib.common.rest_client.RestClient.get',
            fake_result,
            False,
            status=200)

    def test_create_limit(self):
        fake_result = {'foo': 'bar'}
        self.check_service_client_function(
            self.client.create_limit,
            'tempest.lib.common.rest_client.RestClient.post',
            fake_result,
            False,
            region_id='regionOne', service_id='image',
            project_id='project', resource_name='widgets',
            default_limit=10,
            description='Spacely Widgets',
            status=201)

    def test_create_limit_with_domain(self):
        fake_result = {'foo': 'bar'}
        self.check_service_client_function(
            self.client.create_limit,
            'tempest.lib.common.rest_client.RestClient.post',
            fake_result,
            False,
            region_id='regionOne', service_id='image',
            project_id='project', resource_name='widgets',
            default_limit=10,
            domain_id='foo',
            description='Spacely Widgets',
            status=201)

    def test_update_limit(self):
        fake_result = {'foo': 'bar'}
        self.check_service_client_function(
            self.client.update_limit,
            'tempest.lib.common.rest_client.RestClient.patch',
            fake_result,
            False,
            limit_id='123', resource_limit=20,
            status=200)

    def test_update_limit_with_description(self):
        fake_result = {'foo': 'bar'}
        self.check_service_client_function(
            self.client.update_limit,
            'tempest.lib.common.rest_client.RestClient.patch',
            fake_result,
            False,
            limit_id='123', resource_limit=20,
            description='new description',
            status=200)



# Copyright (C) 2017 Dell Inc. or its subsidiaries.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.volume import base
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

CONF = config.CONF


class GroupTypesTest(base.BaseVolumeAdminTest):
    """Test group types"""

    volume_min_microversion = '3.11'
    volume_max_microversion = 'latest'

    @decorators.idempotent_id('dd71e5f9-393e-4d4f-90e9-fa1b8d278864')
    def test_group_type_create_list_update_show_delete(self):
        """Test create/list/update/show/delete group type"""
        name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix,
            name=self.__class__.__name__ + '-group-type')
        description = data_utils.rand_name(
            prefix=CONF.resource_name_prefix,
            name="group-type-description")
        group_specs = {"consistent_group_snapshot_enabled": "<is> False"}
        params = {'name': name,
                  'description': description,
                  'group_specs': group_specs,
                  'is_public': True}
        body = self.admin_group_types_client.create_group_type(
            **params)['group_type']
        self.assertIn('name', body)
        err_msg = ("The created group_type %(var)s is not equal to the "
                   "requested %(var)s")
        self.assertEqual(name, body['name'], err_msg % {"var": "name"})
        self.assertEqual(description, body['description'],
                         err_msg % {"var": "description"})

        group_list = (
            self.admin_group_types_client.list_group_types()['group_types'])
        self.assertIsInstance(group_list, list)
        self.assertNotEmpty(group_list)

        update_params = {
            'name': data_utils.rand_name(
                prefix=CONF.resource_name_prefix,
                name=self.__class__.__name__ + '-updated-group-type'),
            'description': 'updated-group-type-desc'
        }
        updated_group_type = self.admin_group_types_client.update_group_type(
            body['id'], **update_params)['group_type']
        for key, expected_val in update_params.items():
            self.assertEqual(expected_val, updated_group_type[key])

        fetched_group_type = self.admin_group_types_client.show_group_type(
            body['id'])['group_type']
        params.update(update_params)  # Add updated params to original params.
        for key in params.keys():
            self.assertEqual(params[key], fetched_group_type[key],
                             '%s of the fetched group_type is different '
                             'from the created group_type' % key)

        self.admin_group_types_client.delete_group_type(body['id'])
        group_list = (
            self.admin_group_types_client.list_group_types()['group_types'])
        group_ids = [it['id'] for it in group_list]
        self.assertNotIn(body['id'], group_ids)

    @decorators.idempotent_id('3d5e5cec-72b4-4511-b135-7cc2b7a053ae')
    def test_group_type_list_by_optional_params(self):
        """Test list group type sort/public"""
        type_a_name = "a_{}".format(data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='group-type'))
        type_b_name = "b_{}".format(data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='group-type'))
        self.create_group_type(name=type_a_name, **{'is_public': True})
        self.create_group_type(name=type_b_name, **{'is_public': False})

        group_list = (
            self.admin_group_types_client.list_group_types(
                sort="name:asc", is_public=None)['group_types'])
        name_list = [it['name'] for it in group_list]
        self.assertLess(
            name_list.index(type_a_name), name_list.index(type_b_name))

        group_list = (
            self.admin_group_types_client.list_group_types(
                sort="name:desc", is_public=None)['group_types'])
        name_list = [it['name'] for it in group_list]
        self.assertLess(name_list.index(type_b_name),
                        name_list.index(type_a_name))

        group_list = (
            self.admin_group_types_client.list_group_types(
                is_public=False)['group_types'])
        name_list = [it['name'] for it in group_list]
        self.assertNotIn(type_a_name, name_list)
        self.assertIn(type_b_name, name_list)

        group_list = (
            self.admin_group_types_client.list_group_types(
                is_public=True)['group_types'])
        name_list = [it['name'] for it in group_list]
        self.assertNotIn(type_b_name, name_list)
        self.assertIn(type_a_name, name_list)



# Copyright 2014 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.api_schema.response.compute.v2_1 import parameter_types

get_keypair = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'keypair': {
                'type': 'object',
                'properties': {
                    'public_key': {'type': 'string'},
                    'name': {'type': 'string'},
                    'fingerprint': {'type': 'string'},
                    'user_id': {'type': 'string'},
                    'deleted': {'type': 'boolean'},
                    'created_at': parameter_types.date_time,
                    'updated_at': parameter_types.date_time_or_null,
                    'deleted_at': parameter_types.date_time_or_null,
                    'id': {'type': 'integer'}

                },
                'additionalProperties': False,
                'required': ['public_key', 'name', 'fingerprint', 'user_id',
                             'deleted', 'created_at', 'updated_at',
                             'deleted_at', 'id']
            }
        },
        'additionalProperties': False,
        'required': ['keypair']
    }
}

create_keypair = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'keypair': {
                'type': 'object',
                'properties': {
                    'fingerprint': {'type': 'string'},
                    'name': {'type': 'string'},
                    'public_key': {'type': 'string'},
                    'user_id': {'type': 'string'},
                    'private_key': {'type': 'string'}
                },
                'additionalProperties': False,
                # When create keypair API is being called with 'Public key'
                # (Importing keypair) then, response body does not contain
                # 'private_key' So it is not defined as 'required'
                'required': ['fingerprint', 'name', 'public_key', 'user_id']
            }
        },
        'additionalProperties': False,
        'required': ['keypair']
    }
}

delete_keypair = {
    'status_code': [202],
}

list_keypairs = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'keypairs': {
                'type': 'array',
                'items': {
                    'type': 'object',
                    'properties': {
                        'keypair': {
                            'type': 'object',
                            'properties': {
                                'public_key': {'type': 'string'},
                                'name': {'type': 'string'},
                                'fingerprint': {'type': 'string'}
                            },
                            'additionalProperties': False,
                            'required': ['public_key', 'name', 'fingerprint']
                        }
                    },
                    'additionalProperties': False,
                    'required': ['keypair']
                }
            }
        },
        'additionalProperties': False,
        'required': ['keypairs']
    }
}



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


import io
import select
import socket
import time
import warnings

from oslo_log import log as logging
from oslo_utils.secretutils import md5

from tempest.lib import exceptions


with warnings.catch_warnings():
    warnings.simplefilter("ignore")
    import paramiko


LOG = logging.getLogger(__name__)


def get_fingerprint(self):
    """Patch paramiko

    This method needs to be patched to allow paramiko to work under FIPS.
    Until the patch to do this merges, patch paramiko here.

    TODO(alee) Remove this when paramiko is patched.
    See https://github.com/paramiko/paramiko/pull/1928
    """
    return md5(self.asbytes(), usedforsecurity=False).digest()


paramiko.pkey.PKey.get_fingerprint = get_fingerprint


class Client(object):

    def __init__(self, host, username, password=None, timeout=300, pkey=None,
                 channel_timeout=10, look_for_keys=False, key_filename=None,
                 port=22, proxy_client=None, ssh_key_type='rsa',
                 ssh_allow_agent=True):
        """SSH client.

        Many of parameters are just passed to the underlying implementation
        as it is.  See the paramiko documentation for more details.
        http://docs.paramiko.org/en/2.1/api/client.html#paramiko.client.SSHClient.connect

        :param host: Host to login.
        :param username: SSH username.
        :param password: SSH password, or a password to unlock private key.
        :param timeout: Timeout in seconds, including retries.
            Default is 300 seconds.
        :param pkey: Private key.
        :param channel_timeout: Channel timeout in seconds, passed to the
            paramiko.  Default is 10 seconds.
        :param look_for_keys: Whether or not to search for private keys
            in ``~/.ssh``.  Default is False.
        :param key_filename: Filename for private key to use.
        :param port: SSH port number.
        :param proxy_client: Another SSH client to provide a transport
            for ssh-over-ssh.  The default is None, which means
            not to use ssh-over-ssh.
        :param ssh_key_type: ssh key type (rsa, ecdsa)
        :param ssh_allow_agent: boolean, default True, if the SSH client is
            allowed to also utilize the ssh-agent. Explicit use of passwords
            in some tests may need this set as False.
        :type proxy_client: ``tempest.lib.common.ssh.Client`` object
        """
        self.host = host
        self.username = username
        self.port = port
        self.password = password
        if isinstance(pkey, str):
            if ssh_key_type == 'rsa':
                pkey = paramiko.RSAKey.from_private_key(
                    io.StringIO(str(pkey)))
            elif ssh_key_type == 'ecdsa':
                pkey = paramiko.ECDSAKey.from_private_key(
                    io.StringIO(str(pkey)))
            else:
                raise exceptions.SSHClientUnsupportedKeyType(
                    key_type=ssh_key_type)
        self.pkey = pkey
        self.look_for_keys = look_for_keys
        self.key_filename = key_filename
        self.timeout = int(timeout)
        self.channel_timeout = float(channel_timeout)
        self.buf_size = 1024
        self.proxy_client = proxy_client
        if (self.proxy_client and self.proxy_client.host == self.host and
                self.proxy_client.port == self.port and
                self.proxy_client.username == self.username):
            raise exceptions.SSHClientProxyClientLoop(
                host=self.host, port=self.port, username=self.username)
        self._proxy_conn = None
        self.ssh_allow_agent = ssh_allow_agent

    def _get_ssh_connection(self, sleep=1.5, backoff=1):
        """Returns an ssh connection to the specified host."""
        bsleep = sleep
        ssh = paramiko.SSHClient()
        ssh.set_missing_host_key_policy(
            paramiko.AutoAddPolicy())
        _start_time = time.time()
        if self.pkey is not None:
            LOG.info("Creating ssh connection to '%s:%d' as '%s'"
                     " with public key authentication",
                     self.host, self.port, self.username)
        else:
            LOG.info("Creating ssh connection to '%s:%d' as '%s'"
                     " with password %s",
                     self.host, self.port, self.username, str(self.password))
        attempts = 0
        while True:
            if self.proxy_client is not None:
                proxy_chan = self._get_proxy_channel()
            else:
                proxy_chan = None
            try:
                ssh.connect(self.host, port=self.port, username=self.username,
                            password=self.password,
                            look_for_keys=self.look_for_keys,
                            key_filename=self.key_filename,
                            timeout=self.channel_timeout, pkey=self.pkey,
                            sock=proxy_chan, allow_agent=self.ssh_allow_agent)
                LOG.info("ssh connection to %s@%s successfully created",
                         self.username, self.host)
                return ssh
            except (EOFError,
                    socket.error, socket.timeout,
                    paramiko.SSHException) as e:
                ssh.close()
                if self._is_timed_out(_start_time):
                    LOG.exception("Failed to establish authenticated ssh"
                                  " connection to %s@%s after %d attempts. "
                                  "Proxy client: %s",
                                  self.username, self.host, attempts,
                                  self._get_proxy_client_info())
                    raise exceptions.SSHTimeout(host=self.host,
                                                user=self.username,
                                                password=self.password)
                bsleep += backoff
                attempts += 1
                LOG.warning("Failed to establish authenticated ssh"
                            " connection to %s@%s (%s). Number attempts: %s."
                            " Retry after %d seconds.",
                            self.username, self.host, e, attempts, bsleep)
                time.sleep(bsleep)

    def _is_timed_out(self, start_time):
        return (time.time() - self.timeout) > start_time

    @staticmethod
    def _can_system_poll():
        return hasattr(select, 'poll')

    def exec_command(self, cmd, encoding="utf-8"):
        """Execute the specified command on the server

        Note that this method is reading whole command outputs to memory, thus
        shouldn't be used for large outputs.

        :param str cmd: Command to run at remote server.
        :param str encoding: Encoding for result from paramiko.
                             Result will not be decoded if None.
        :returns: data read from standard output of the command.
        :raises: SSHExecCommandFailed if command returns nonzero
                 status. The exception contains command status stderr content.
        :raises: TimeoutException if cmd doesn't end when timeout expires.
        """
        ssh = self._get_ssh_connection()
        transport = ssh.get_transport()
        with transport.open_session() as channel:
            channel.fileno()  # Register event pipe
            channel.exec_command(cmd)
            channel.shutdown_write()

            # If the executing host is linux-based, poll the channel
            if self._can_system_poll():
                out_data_chunks = []
                err_data_chunks = []
                poll = select.poll()
                poll.register(channel, select.POLLIN)
                start_time = time.time()

                while True:
                    ready = poll.poll(self.channel_timeout)
                    if not any(ready):
                        if not self._is_timed_out(start_time):
                            continue
                        raise exceptions.TimeoutException(
                            "Command: '{0}' executed on host '{1}'.".format(
                                cmd, self.host))
                    if not ready[0]:  # If there is nothing to read.
                        continue
                    out_chunk = err_chunk = None
                    if channel.recv_ready():
                        out_chunk = channel.recv(self.buf_size)
                        out_data_chunks += out_chunk,
                    if channel.recv_stderr_ready():
                        err_chunk = channel.recv_stderr(self.buf_size)
                        err_data_chunks += err_chunk,
                    if not err_chunk and not out_chunk:
                        break
                out_data = b''.join(out_data_chunks)
                err_data = b''.join(err_data_chunks)
            # Just read from the channels
            else:
                out_file = channel.makefile('rb', self.buf_size)
                err_file = channel.makefile_stderr('rb', self.buf_size)
                out_data = out_file.read()
                err_data = err_file.read()
            if encoding:
                out_data = out_data.decode(encoding)
                err_data = err_data.decode(encoding)

            exit_status = channel.recv_exit_status()

        ssh.close()

        if 0 != exit_status:
            raise exceptions.SSHExecCommandFailed(
                command=cmd, exit_status=exit_status,
                stderr=err_data, stdout=out_data)
        return out_data

    def test_connection_auth(self):
        """Raises an exception when we can not connect to server via ssh."""
        connection = self._get_ssh_connection()
        connection.close()

    def _get_proxy_channel(self):
        conn = self.proxy_client._get_ssh_connection()
        # Keep a reference to avoid g/c
        # https://github.com/paramiko/paramiko/issues/440
        self._proxy_conn = conn
        transport = conn.get_transport()
        chan = transport.open_session()
        cmd = 'nc %s %s' % (self.host, self.port)
        chan.exec_command(cmd)
        return chan

    def _get_proxy_client_info(self):
        if not self.proxy_client:
            return 'no proxy client'
        nested_pclient = self.proxy_client._get_proxy_client_info()
        return ('%(username)s@%(host)s:%(port)s, nested proxy client: '
                '%(nested_pclient)s' % {'username': self.proxy_client.username,
                                        'host': self.proxy_client.host,
                                        'port': self.proxy_client.port,
                                        'nested_pclient': nested_pclient})



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_log import log as logging
import testtools
from testtools import matchers

from tempest.api.compute import base
from tempest.common import identity
from tempest.common import tempest_fixtures as fixtures
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

CONF = config.CONF
LOG = logging.getLogger(__name__)


class QuotasAdminTestBase(base.BaseV2ComputeAdminTest):
    force_tenant_isolation = True

    def setUp(self):
        # NOTE(mriedem): Avoid conflicts with os-quota-class-sets tests.
        self.useFixture(fixtures.LockFixture('compute_quotas'))
        super(QuotasAdminTestBase, self).setUp()

    @classmethod
    def setup_clients(cls):
        super(QuotasAdminTestBase, cls).setup_clients()
        cls.adm_client = cls.os_admin.quotas_client

    def _get_updated_quotas(self):
        # Verify that GET shows the updated quota set of project
        project_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix,
            name='cpu_quota_project')
        project_desc = project_name + '-desc'
        project = identity.identity_utils(self.os_admin).create_project(
            name=project_name, description=project_desc)
        project_id = project['id']
        self.addCleanup(identity.identity_utils(self.os_admin).delete_project,
                        project_id)

        self.adm_client.update_quota_set(project_id, ram='5120')
        # Call show_quota_set with detail=true to cover the
        # get_quota_set_details response schema for microversion tests
        quota_set = self.adm_client.show_quota_set(
            project_id, detail=True)['quota_set']
        self.assertEqual(5120, quota_set['ram']['limit'])

        # Verify that GET shows the updated quota set of user
        user_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix,
            name='cpu_quota_user')
        password = data_utils.rand_password()
        email = user_name + '@testmail.tm'
        user = identity.identity_utils(self.os_admin).create_user(
            username=user_name, password=password, project=project,
            email=email)
        user_id = user['id']
        self.addCleanup(identity.identity_utils(self.os_admin).delete_user,
                        user_id)

        self.adm_client.update_quota_set(project_id,
                                         user_id=user_id,
                                         ram='2048')
        quota_set = self.adm_client.show_quota_set(
            project_id, user_id=user_id)['quota_set']
        self.assertEqual(2048, quota_set['ram'])

    @classmethod
    def resource_setup(cls):
        super(QuotasAdminTestBase, cls).resource_setup()

        # NOTE(afazekas): these test cases should always create and use a new
        # tenant most of them should be skipped if we can't do that
        cls.demo_tenant_id = cls.quotas_client.tenant_id

        cls.default_quota_set = set(('metadata_items', 'ram', 'key_pairs',
                                     'instances', 'cores',
                                     'server_group_members', 'server_groups'))
        if cls.is_requested_microversion_compatible('2.35'):
            cls.default_quota_set = \
                cls.default_quota_set | set(['fixed_ips', 'floating_ips',
                                             'security_group_rules',
                                             'security_groups'])
        if cls.is_requested_microversion_compatible('2.56'):
            cls.default_quota_set = \
                cls.default_quota_set | set(['injected_file_content_bytes',
                                             'injected_file_path_bytes',
                                             'injected_files'])


class QuotasAdminTestJSON(QuotasAdminTestBase):
    """Test compute quotas by admin user"""

    @decorators.idempotent_id('3b0a7c8f-cf58-46b8-a60c-715a32a8ba7d')
    def test_get_default_quotas(self):
        """Test admin can get the default compute quota set for a project"""
        expected_quota_set = self.default_quota_set | set(['id'])
        quota_set = self.adm_client.show_default_quota_set(
            self.demo_tenant_id)['quota_set']
        self.assertEqual(quota_set['id'], self.demo_tenant_id)
        for quota in expected_quota_set:
            self.assertIn(quota, quota_set.keys())

    @decorators.idempotent_id('55fbe2bf-21a9-435b-bbd2-4162b0ed799a')
    @testtools.skipIf(CONF.compute_feature_enabled.unified_limits,
                      'Legacy quota update not available with unified limits')
    def test_update_all_quota_resources_for_tenant(self):
        """Test admin can update all the compute quota limits for a project"""
        default_quota_set = self.adm_client.show_default_quota_set(
            self.demo_tenant_id)['quota_set']
        new_quota_set = {'metadata_items': 256, 'ram': 10240,
                         'key_pairs': 200, 'instances': 20,
                         'server_groups': 20,
                         'server_group_members': 20, 'cores': 2}
        if self.is_requested_microversion_compatible('2.35'):
            new_quota_set.update({'fixed_ips': 10, 'floating_ips': 20,
                                  'security_group_rules': 20,
                                  'security_groups': 20})
        if self.is_requested_microversion_compatible('2.56'):
            new_quota_set.update({'injected_file_content_bytes': 20480,
                                  'injected_file_path_bytes': 512,
                                  'injected_files': 10})

        # Update limits for all quota resources
        quota_set = self.adm_client.update_quota_set(
            self.demo_tenant_id,
            force=True,
            **new_quota_set)['quota_set']

        default_quota_set.pop('id')
        self.addCleanup(self.adm_client.update_quota_set,
                        self.demo_tenant_id, **default_quota_set)
        for quota in new_quota_set:
            self.assertIn(quota, quota_set.keys())

    # TODO(afazekas): merge these test cases
    @decorators.idempotent_id('ce9e0815-8091-4abd-8345-7fe5b85faa1d')
    @testtools.skipIf(CONF.compute_feature_enabled.unified_limits,
                      'Legacy quota update not available with unified limits')
    def test_get_updated_quotas(self):
        """Test that GET shows the updated quota set of project"""
        self._get_updated_quotas()

    @decorators.idempotent_id('389d04f0-3a41-405f-9317-e5f86e3c44f0')
    @testtools.skipIf(CONF.compute_feature_enabled.unified_limits,
                      'Legacy quota update not available with unified limits')
    def test_delete_quota(self):
        """Test admin can delete the compute quota set for a project"""
        project_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix,
            name='ram_quota_project')
        project_desc = project_name + '-desc'
        project = identity.identity_utils(self.os_admin).create_project(
            name=project_name, description=project_desc)
        project_id = project['id']
        self.addCleanup(identity.identity_utils(self.os_admin).delete_project,
                        project_id)
        quota_set_default = (self.adm_client.show_quota_set(project_id)
                             ['quota_set'])
        ram_default = quota_set_default['ram']

        self.adm_client.update_quota_set(project_id, ram='5120')

        self.adm_client.delete_quota_set(project_id)

        quota_set_new = self.adm_client.show_quota_set(project_id)['quota_set']
        self.assertEqual(ram_default, quota_set_new['ram'])


class QuotasAdminTestV236(QuotasAdminTestBase):
    """Test compute quotas with microversion greater than 2.35

    # NOTE(gmann): This test tests the Quota APIs response schema
    # for 2.36 microversion. No specific assert or behaviour verification
    # is needed.
    """

    min_microversion = '2.36'

    @decorators.idempotent_id('4268b5c9-92e5-4adc-acf1-3a2798f3d803')
    @testtools.skipIf(CONF.compute_feature_enabled.unified_limits,
                      'Legacy quota update not available with unified limits')
    def test_get_updated_quotas(self):
        """Test compute quotas API with microversion greater than 2.35

        Checking compute quota update, get, get details APIs response schema.
        """
        self._get_updated_quotas()


class QuotasAdminTestV257(QuotasAdminTestBase):
    """Test compute quotas with microversion greater than 2.56

    # NOTE(gmann): This test tests the Quota APIs response schema
    # for 2.57 microversion. No specific assert or behaviour verification
    # is needed.
    """

    min_microversion = '2.57'

    @decorators.idempotent_id('e641e6c6-e86c-41a4-9e5c-9493c0ae47ad')
    @testtools.skipIf(CONF.compute_feature_enabled.unified_limits,
                      'Legacy quota update not available with unified limits')
    def test_get_updated_quotas(self):
        """Test compute quotas API with microversion greater than 2.56

        Checking compute quota update, get, get details APIs response schema.
        """
        self._get_updated_quotas()


class QuotaClassesAdminTestJSON(base.BaseV2ComputeAdminTest):
    """Tests the os-quota-class-sets API to update default quotas."""

    def setUp(self):
        # All test cases in this class need to externally lock on doing
        # anything with default quota values.
        self.useFixture(fixtures.LockFixture('compute_quotas'))
        super(QuotaClassesAdminTestJSON, self).setUp()

    @classmethod
    def resource_setup(cls):
        super(QuotaClassesAdminTestJSON, cls).resource_setup()
        cls.adm_client = cls.os_admin.quota_classes_client

    def _restore_default_quotas(self, original_defaults):
        LOG.debug("restoring quota class defaults")
        self.adm_client.update_quota_class_set('default', **original_defaults)

    # NOTE(sdague): this test is problematic as it changes
    # global state, and possibly needs to be part of a set of
    # tests that get run all by themselves at the end under a
    # 'danger' flag.
    @decorators.idempotent_id('7932ab0f-5136-4075-b201-c0e2338df51a')
    @testtools.skipIf(CONF.compute_feature_enabled.unified_limits,
                      'Legacy quota update not available with unified limits')
    def test_update_default_quotas(self):
        """Test updating default compute quota class set"""
        # get the current 'default' quota class values
        body = (self.adm_client.show_quota_class_set('default')
                ['quota_class_set'])
        self.assertEqual('default', body.pop('id'))
        # restore the defaults when the test is done
        self.addCleanup(self._restore_default_quotas, body.copy())
        # increment all of the values for updating the default quota class
        for quota, default in body.items():
            # NOTE(sdague): we need to increment a lot, otherwise
            # there is a real chance that we go from -1 (unlimited)
            # to a very small number which causes issues.
            body[quota] = default + 100
        # update limits for the default quota class set
        update_body = self.adm_client.update_quota_class_set(
            'default', **body)['quota_class_set']
        # assert that the response has all of the changed values
        self.assertThat(update_body.items(),
                        matchers.ContainsAll(body.items()))
        # check quota values are changed
        show_body = self.adm_client.show_quota_class_set(
            'default')['quota_class_set']
        self.assertThat(show_body.items(),
                        matchers.ContainsAll(body.items()))


class QuotaClassesAdmin257Test(QuotaClassesAdminTestJSON):
    """Test compute quotas with microversion greater than 2.56

    # NOTE(gmann): This test tests the Quota class APIs response schema
    # for 2.57 microversion. No specific assert or behaviour verification
    # is needed.
    """

    min_microversion = '2.57'



# Copyright 2016 NEC Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.compute.keypairs import base
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

CONF = config.CONF


class KeyPairsV210TestJSON(base.BaseKeypairTest):
    """Tests KeyPairs API with microversion higher than 2.9"""

    credentials = ['primary', 'admin']
    min_microversion = '2.10'

    @classmethod
    def setup_clients(cls):
        super(KeyPairsV210TestJSON, cls).setup_clients()
        cls.client = cls.os_admin.keypairs_client
        cls.non_admin_client = cls.os_primary.keypairs_client

    def _create_and_check_keypairs(self, user_id):
        key_list = list()
        for _ in range(2):
            k_name = data_utils.rand_name(
                prefix=CONF.resource_name_prefix, name='keypair')
            keypair = self.create_keypair(k_name,
                                          keypair_type='ssh',
                                          user_id=user_id,
                                          client=self.client)
            self.assertEqual(k_name, keypair['name'],
                             "The created keypair name is not equal "
                             "to the requested name!")
            self.assertEqual(user_id, keypair['user_id'],
                             "The created keypair is not for requested user!")
            keypair.pop('private_key', None)
            keypair.pop('user_id')
            key_list.append(keypair)
        return key_list

    @decorators.idempotent_id('3c8484af-cfb3-48f6-b8ba-d5d58bbf3eac')
    def test_admin_manage_keypairs_for_other_users(self):
        """Test admin managing keypairs for other users

        First admin creates a keypair for an other user, then admin lists
        keypairs filtered by that user, and keypairs created for that user
        should appear in the result and keypairs not created for that user
        should not appear in the result.
        """
        user_id = self.non_admin_client.user_id
        key_list = self._create_and_check_keypairs(user_id)
        first_keyname = key_list[0]['name']
        keypair_detail = self.client.show_keypair(first_keyname,
                                                  user_id=user_id)['keypair']
        self.assertEqual(first_keyname, keypair_detail['name'])
        self.assertEqual(user_id, keypair_detail['user_id'],
                         "The fetched keypair is not for requested user!")
        # Create an admin keypair
        admin_keypair = self.create_keypair(keypair_type='ssh',
                                            client=self.client)
        admin_keypair.pop('private_key', None)
        admin_keypair.pop('user_id')

        # Admin fetch keypairs list of non admin user
        keypairs = self.client.list_keypairs(user_id=user_id)['keypairs']
        fetched_list = [keypair['keypair'] for keypair in keypairs]

        # Check admin keypair is not present in non admin user keypairs list
        self.assertNotIn(admin_keypair, fetched_list,
                         "The fetched user keypairs has admin keypair!")

        # Now check if all the created keypairs are in the fetched list
        missing_kps = [kp for kp in key_list if kp not in fetched_list]
        self.assertFalse(missing_kps,
                         "Failed to find keypairs %s in fetched list"
                         % ', '.join(m_key['name'] for m_key in missing_kps))



# Copyright 2017 NEC Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from tempest.lib.common import ssh
from tempest.lib.common.utils.linux import remote_client
from tempest.lib import exceptions as lib_exc
from tempest.tests import base


class FakeServersClient(object):

    def get_console_output(self, server_id):
        return {"output": "fake_output"}


class TestRemoteClient(base.TestCase):

    @mock.patch.object(ssh.Client, 'exec_command', return_value='success')
    def test_exec_command(self, mock_ssh_exec_command):
        client = remote_client.RemoteClient('192.168.1.10', 'username')
        client.exec_command('ls')
        mock_ssh_exec_command.assert_called_once_with(
            'set -eu -o pipefail; PATH=$PATH:/sbin; ls')

    @mock.patch.object(ssh.Client, 'test_connection_auth')
    def test_validate_authentication(self, mock_test_connection_auth):
        client = remote_client.RemoteClient('192.168.1.10', 'username')
        client.validate_authentication()
        mock_test_connection_auth.assert_called_once_with()

    @mock.patch.object(remote_client.LOG, 'debug')
    @mock.patch.object(ssh.Client, 'exec_command')
    def test_debug_ssh_without_console(self, mock_exec_command, mock_debug):
        mock_exec_command.side_effect = lib_exc.SSHTimeout
        server = {'id': 'fake_id'}
        client = remote_client.RemoteClient('192.168.1.10', 'username',
                                            server=server)
        self.assertRaises(lib_exc.SSHTimeout, client.exec_command, 'ls')
        mock_debug.assert_called_with(
            'Caller: %s. Timeout trying to ssh to server %s',
            'TestRemoteClient:test_debug_ssh_without_console', server)

    @mock.patch.object(remote_client.LOG, 'debug')
    @mock.patch.object(ssh.Client, 'exec_command')
    def test_debug_ssh_with_console(self, mock_exec_command, mock_debug):
        mock_exec_command.side_effect = lib_exc.SSHTimeout
        server = {'id': 'fake_id'}
        client = remote_client.RemoteClient('192.168.1.10', 'username',
                                            server=server,
                                            servers_client=FakeServersClient())
        self.assertRaises(lib_exc.SSHTimeout, client.exec_command, 'ls')
        mock_debug.assert_called_with(
            'Console log for server %s: %s', server['id'], 'fake_output')



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from urllib import parse as urllib

from oslo_serialization import jsonutils as json

from tempest.lib.api_schema.response.compute.v2_1 import baremetal_nodes \
    as schema
from tempest.lib.common import rest_client
from tempest.lib.services.compute import base_compute_client


class BaremetalNodesClient(base_compute_client.BaseComputeClient):
    """Tests Baremetal API"""

    def list_baremetal_nodes(self, **params):
        """List all baremetal nodes.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#list-bare-metal-nodes
        """
        url = 'os-baremetal-nodes'
        if params:
            url += '?%s' % urllib.urlencode(params)
        resp, body = self.get(url)
        body = json.loads(body)
        self.validate_response(schema.list_baremetal_nodes, resp, body)
        return rest_client.ResponseBody(resp, body)

    def show_baremetal_node(self, baremetal_node_id):
        """Show the details of a single baremetal node.

        For more information, please refer to the official API reference:
        https://docs.openstack.org/api-ref/compute/#show-bare-metal-node-details
        """
        url = 'os-baremetal-nodes/%s' % baremetal_node_id
        resp, body = self.get(url)
        body = json.loads(body)
        self.validate_response(schema.get_baremetal_node, resp, body)
        return rest_client.ResponseBody(resp, body)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.services.network import base


class TrunksClient(base.BaseNetworkClient):

    def create_trunk(self, **kwargs):
        """Creates a trunk.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#create-trunk
        """
        uri = '/trunks'
        post_data = {'trunk': kwargs}
        return self.create_resource(uri, post_data)

    def update_trunk(self, trunk_id, **kwargs):
        """Updates a trunk.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#update-trunk
        """
        uri = '/trunks/%s' % trunk_id
        put_data = {'trunk': kwargs}
        return self.update_resource(uri, put_data)

    def show_trunk(self, trunk_id):
        """Shows details for a trunk.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#show-trunk
        """
        uri = '/trunks/%s' % trunk_id
        return self.show_resource(uri)

    def delete_trunk(self, trunk_id):
        """Deletes a trunk.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#delete-trunk
        """
        uri = '/trunks/%s' % trunk_id
        return self.delete_resource(uri)

    def list_trunks(self, **filters):
        """Lists trunks to which the tenant has access.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#list-trunks
        """
        uri = '/trunks'
        return self.list_resources(uri, **filters)

    def add_subports_to_trunk(self, trunk_id, sub_ports):
        """Add subports to a trunk.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#add-subports-to-trunk
        """
        uri = '/trunks/%s/add_subports' % trunk_id
        put_data = {'sub_ports': sub_ports}
        return self.update_resource(uri, put_data)

    def delete_subports_from_trunk(self, trunk_id, sub_ports):
        """Deletes subports from a trunk.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#delete-subports-from-trunk
        """
        uri = '/trunks/%s/remove_subports' % trunk_id
        put_data = {'sub_ports': sub_ports}
        return self.update_resource(uri, put_data)

    def list_subports_of_trunk(self, trunk_id):
        """List subports of a trunk.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#list-subports-for-trunk
        """
        uri = '/trunks/%s/get_subports' % trunk_id
        return self.list_resources(uri)



# Copyright 2013 Huawei Technologies Co.,LTD
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from testtools import matchers

from tempest.api.volume import base
from tempest import config
from tempest.lib import decorators

CONF = config.CONF


class SnapshotMetadataTestJSON(base.BaseVolumeTest):
    """Test snapshot metadata"""

    @classmethod
    def skip_checks(cls):
        super(SnapshotMetadataTestJSON, cls).skip_checks()
        if not CONF.volume_feature_enabled.snapshot:
            raise cls.skipException("Cinder snapshot feature disabled")

    @classmethod
    def resource_setup(cls):
        super(SnapshotMetadataTestJSON, cls).resource_setup()
        # Create a volume
        cls.volume = cls.create_volume()
        # Create a snapshot
        cls.snapshot = cls.create_snapshot(volume_id=cls.volume['id'])

    def tearDown(self):
        # Update the metadata to {}
        self.snapshots_client.update_snapshot_metadata(
            self.snapshot['id'], metadata={})
        super(SnapshotMetadataTestJSON, self).tearDown()

    @decorators.idempotent_id('a2f20f99-e363-4584-be97-bc33afb1a56c')
    def test_crud_snapshot_metadata(self):
        """Test create/get/update/delete snapshot metadata"""
        # Create metadata for the snapshot
        metadata = {"key1": "value1",
                    "key2": "value2",
                    "key3": "value3"}
        update = {"key3": "value3_update",
                  "key4": "value4"}
        expect = {"key4": "value4"}
        # Create metadata
        body = self.snapshots_client.create_snapshot_metadata(
            self.snapshot['id'], metadata)['metadata']
        self.assertThat(body.items(), matchers.ContainsAll(metadata.items()))

        # Get the metadata of the snapshot
        body = self.snapshots_client.show_snapshot_metadata(
            self.snapshot['id'])['metadata']
        self.assertThat(body.items(), matchers.ContainsAll(metadata.items()),
                        'Create snapshot metadata failed')

        # Update metadata
        body = self.snapshots_client.update_snapshot_metadata(
            self.snapshot['id'], metadata=update)['metadata']
        self.assertEqual(update, body)
        body = self.snapshots_client.show_snapshot_metadata(
            self.snapshot['id'])['metadata']
        self.assertEqual(update, body, 'Update snapshot metadata failed')

        # Delete one item metadata of the snapshot
        self.snapshots_client.delete_snapshot_metadata_item(
            self.snapshot['id'], "key3")
        body = self.snapshots_client.show_snapshot_metadata(
            self.snapshot['id'])['metadata']
        self.assertThat(body.items(), matchers.ContainsAll(expect.items()),
                        'Delete one item metadata of the snapshot failed')
        self.assertNotIn("key3", body)

    @decorators.idempotent_id('e8ff85c5-8f97-477f-806a-3ac364a949ed')
    def test_update_show_snapshot_metadata_item(self):
        """Test update/show snapshot metadata item"""
        metadata = {"key1": "value1",
                    "key2": "value2",
                    "key3": "value3"}
        update_item = {"key3": "value3_update"}
        expect = {"key1": "value1",
                  "key2": "value2",
                  "key3": "value3_update"}
        # Create metadata for the snapshot
        self.snapshots_client.create_snapshot_metadata(
            self.snapshot['id'], metadata)
        # Get the metadata of the snapshot
        body = self.snapshots_client.show_snapshot_metadata(
            self.snapshot['id'])['metadata']
        self.assertThat(body.items(), matchers.ContainsAll(metadata.items()))
        # Update metadata item
        body = self.snapshots_client.update_snapshot_metadata_item(
            self.snapshot['id'], "key3", meta=update_item)['meta']
        self.assertEqual(update_item, body)

        # Get a specific metadata item of the snapshot
        body = self.snapshots_client.show_snapshot_metadata_item(
            self.snapshot['id'], "key3")['meta']
        self.assertEqual({"key3": expect['key3']}, body)

        # Get the metadata of the snapshot
        body = self.snapshots_client.show_snapshot_metadata(
            self.snapshot['id'])['metadata']
        self.assertThat(body.items(), matchers.ContainsAll(expect.items()))



# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_serialization import jsonutils as json

from tempest.lib.api_schema.response.volume import qos as schema
from tempest.lib.common import rest_client
from tempest.lib import exceptions as lib_exc


class QosSpecsClient(rest_client.RestClient):
    """Volume QoS client.

       Client class to send CRUD QoS API requests
    """

    def is_resource_deleted(self, qos_id):
        try:
            self.show_qos(qos_id)
        except lib_exc.NotFound:
            return True
        return False

    @property
    def resource_type(self):
        """Returns the primary type of resource this client works with."""
        return 'qos'

    def create_qos(self, **kwargs):
        """Create a QoS Specification.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v3/index.html#create-a-qos-specification
        """
        post_body = json.dumps({'qos_specs': kwargs})
        resp, body = self.post('qos-specs', post_body)
        body = json.loads(body)
        self.validate_response(schema.show_qos, resp, body)
        return rest_client.ResponseBody(resp, body)

    def delete_qos(self, qos_id, force=False):
        """Delete the specified QoS specification."""
        resp, body = self.delete(
            "qos-specs/%s?force=%s" % (qos_id, force))
        self.validate_response(schema.delete_qos, resp, body)
        return rest_client.ResponseBody(resp, body)

    def list_qos(self):
        """List all the QoS specifications created."""
        url = 'qos-specs'
        resp, body = self.get(url)
        body = json.loads(body)
        self.validate_response(schema.list_qos, resp, body)
        return rest_client.ResponseBody(resp, body)

    def show_qos(self, qos_id):
        """Get the specified QoS specification."""
        url = "qos-specs/%s" % qos_id
        resp, body = self.get(url)
        body = json.loads(body)
        self.validate_response(schema.show_qos, resp, body)
        return rest_client.ResponseBody(resp, body)

    def set_qos_key(self, qos_id, **kwargs):
        """Set the specified keys/values of QoS specification.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v3/index.html#set-keys-in-a-qos-specification
        """
        put_body = json.dumps({"qos_specs": kwargs})
        resp, body = self.put('qos-specs/%s' % qos_id, put_body)
        body = json.loads(body)
        self.validate_response(schema.set_qos_key, resp, body)
        return rest_client.ResponseBody(resp, body)

    def unset_qos_key(self, qos_id, keys):
        """Unset the specified keys of QoS specification.

        :param keys: keys to delete from the QoS specification.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v3/index.html#unset-keys-in-a-qos-specification
        """
        put_body = json.dumps({'keys': keys})
        resp, body = self.put('qos-specs/%s/delete_keys' % qos_id, put_body)
        self.validate_response(schema.unset_qos_key, resp, body)
        return rest_client.ResponseBody(resp, body)

    def associate_qos(self, qos_id, vol_type_id):
        """Associate the specified QoS with specified volume-type."""
        url = "qos-specs/%s/associate" % qos_id
        url += "?vol_type_id=%s" % vol_type_id
        resp, body = self.get(url)
        self.validate_response(schema.associate_qos, resp, body)
        return rest_client.ResponseBody(resp, body)

    def show_association_qos(self, qos_id):
        """Get the association of the specified QoS specification."""
        url = "qos-specs/%s/associations" % qos_id
        resp, body = self.get(url)
        body = json.loads(body)
        self.validate_response(schema.show_association_qos, resp, body)
        return rest_client.ResponseBody(resp, body)

    def disassociate_qos(self, qos_id, vol_type_id):
        """Disassociate the specified QoS with specified volume-type."""
        url = "qos-specs/%s/disassociate" % qos_id
        url += "?vol_type_id=%s" % vol_type_id
        resp, body = self.get(url)
        self.validate_response(schema.disassociate_qos, resp, body)
        return rest_client.ResponseBody(resp, body)

    def disassociate_all_qos(self, qos_id):
        """Disassociate the specified QoS with all associations."""
        url = "qos-specs/%s/disassociate_all" % qos_id
        resp, body = self.get(url)
        self.validate_response(schema.disassociate_all_qos, resp, body)
        return rest_client.ResponseBody(resp, body)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

from tempest.lib.services.network import qos_minimum_packet_rate_rules_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestQosMinimumPacketRateRulesClient(base.BaseServiceTest):

    FAKE_QOS_POLICY_ID = "f1011b08-1297-11e9-a1e7-c7e6825a2616"
    FAKE_MIN_PPS_RULE_ID = "e758c89e-1297-11e9-a6cf-cf46a71e6699"

    FAKE_MIN_PPS_RULE_REQUEST = {
        'qos_policy_id': FAKE_QOS_POLICY_ID,
        'min_kpps': 1000,
        'direction': 'ingress'
    }

    FAKE_MIN_PPS_RULE_RESPONSE = {
        'minimum_packet_rate_rule': {
            'id': FAKE_MIN_PPS_RULE_ID,
            'min_kpps': 1000,
            'direction': 'ingress'
        }
    }

    FAKE_MIN_PPS_RULES = {
        'minimum_packet_rate_rules': [
            FAKE_MIN_PPS_RULE_RESPONSE['minimum_packet_rate_rule']
        ]
    }

    def setUp(self):
        super(TestQosMinimumPacketRateRulesClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.qos_min_pps_client = qos_minimum_packet_rate_rules_client.\
            QosMinimumPacketRateRulesClient(fake_auth, "network", "regionOne")

    def _test_create_minimum_packet_rate_rule(self, bytes_body=False):
        self.check_service_client_function(
            self.qos_min_pps_client.create_minimum_packet_rate_rule,
            "tempest.lib.common.rest_client.RestClient.post",
            self.FAKE_MIN_PPS_RULE_RESPONSE,
            bytes_body,
            201,
            **self.FAKE_MIN_PPS_RULE_REQUEST
        )

    def _test_list_minimum_packet_rate_rules(self, bytes_body=False):
        self.check_service_client_function(
            self.qos_min_pps_client.list_minimum_packet_rate_rules,
            "tempest.lib.common.rest_client.RestClient.get",
            self.FAKE_MIN_PPS_RULES,
            bytes_body,
            200,
            qos_policy_id=self.FAKE_QOS_POLICY_ID
        )

    def _test_show_minimum_packet_rate_rule(self, bytes_body=False):
        self.check_service_client_function(
            self.qos_min_pps_client.show_minimum_packet_rate_rule,
            "tempest.lib.common.rest_client.RestClient.get",
            self.FAKE_MIN_PPS_RULE_RESPONSE,
            bytes_body,
            200,
            qos_policy_id=self.FAKE_QOS_POLICY_ID,
            rule_id=self.FAKE_MIN_PPS_RULE_ID
        )

    def _test_update_qos_polcy(self, bytes_body=False):
        update_kwargs = {
            "min_kpps": "20000"
        }

        resp_body = {
            "minimum_packet_rate_rule": copy.deepcopy(
                self.FAKE_MIN_PPS_RULE_RESPONSE['minimum_packet_rate_rule']
            )
        }
        resp_body["minimum_packet_rate_rule"].update(update_kwargs)

        self.check_service_client_function(
            self.qos_min_pps_client.update_minimum_packet_rate_rule,
            "tempest.lib.common.rest_client.RestClient.put",
            resp_body,
            bytes_body,
            200,
            qos_policy_id=self.FAKE_QOS_POLICY_ID,
            rule_id=self.FAKE_MIN_PPS_RULE_ID,
            **update_kwargs)

    def test_create_minimum_packet_rate_rule_with_str_body(self):
        self._test_create_minimum_packet_rate_rule()

    def test_create_minimum_packet_rate_rule_with_bytes_body(self):
        self._test_create_minimum_packet_rate_rule(bytes_body=True)

    def test_update_minimum_packet_rate_rule_with_str_body(self):
        self._test_update_qos_polcy()

    def test_update_minimum_packet_rate_rule_with_bytes_body(self):
        self._test_update_qos_polcy(bytes_body=True)

    def test_show_minimum_packet_rate_rule_with_str_body(self):
        self._test_show_minimum_packet_rate_rule()

    def test_show_minimum_packet_rate_rule_with_bytes_body(self):
        self._test_show_minimum_packet_rate_rule(bytes_body=True)

    def test_delete_minimum_packet_rate_rule(self):
        self.check_service_client_function(
            self.qos_min_pps_client.delete_minimum_packet_rate_rule,
            "tempest.lib.common.rest_client.RestClient.delete",
            {},
            status=204,
            qos_policy_id=self.FAKE_QOS_POLICY_ID,
            rule_id=self.FAKE_MIN_PPS_RULE_ID)

    def test_list_minimum_packet_rate_rule_with_str_body(self):
        self._test_list_minimum_packet_rate_rules()

    def test_list_minimum_packet_rate_rule_with_bytes_body(self):
        self._test_list_minimum_packet_rate_rules(bytes_body=True)



# Copyright 2013 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import testtools

from tempest.api.identity import base
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

CONF = config.CONF


class ProjectsTestJSON(base.BaseIdentityV3AdminTest):
    """Test identity projects"""

    # NOTE: force_tenant_isolation is true in the base class by default but
    # overridden to false here to allow test execution for clouds using the
    # pre-provisioned credentials provider.
    force_tenant_isolation = False

    @decorators.idempotent_id('0ecf465c-0dc4-4532-ab53-91ffeb74d12d')
    def test_project_create_with_description(self):
        """Test creating project with a description"""
        project_desc = data_utils.rand_name(
            name='desc', prefix=CONF.resource_name_prefix)
        project = self.setup_test_project(description=project_desc)
        project_id = project['id']
        desc1 = project['description']
        self.assertEqual(desc1, project_desc, 'Description should have '
                         'been sent in response for create')
        body = self.projects_client.show_project(project_id)['project']
        desc2 = body['description']
        self.assertEqual(desc2, project_desc, 'Description does not appear '
                         'to be set')

    @decorators.idempotent_id('5f50fe07-8166-430b-a882-3b2ee0abe26f')
    def test_project_create_with_domain(self):
        """Test creating project with a domain"""
        domain = self.setup_test_domain()
        project_name = data_utils.rand_name(
            name='project', prefix=CONF.resource_name_prefix)
        project = self.setup_test_project(
            name=project_name, domain_id=domain['id'])
        project_id = project['id']
        self.assertEqual(project_name, project['name'])
        self.assertEqual(domain['id'], project['domain_id'])
        body = self.projects_client.show_project(project_id)['project']
        self.assertEqual(project_name, body['name'])
        self.assertEqual(domain['id'], body['domain_id'])

    @decorators.idempotent_id('1854f9c0-70bc-4d11-a08a-1c789d339e3d')
    def test_project_create_with_parent(self):
        """Test creating root project without providing a parent_id"""
        domain = self.setup_test_domain()
        domain_id = domain['id']

        root_project_name = data_utils.rand_name(
            name='root_project', prefix=CONF.resource_name_prefix)
        root_project = self.setup_test_project(
            name=root_project_name, domain_id=domain_id)

        root_project_id = root_project['id']
        parent_id = root_project['parent_id']
        self.assertEqual(root_project_name, root_project['name'])
        # If not provided, the parent_id must point to the top level
        # project in the hierarchy, i.e. its domain
        self.assertEqual(domain_id, parent_id)

        # Create a project using root_project_id as parent_id
        project_name = data_utils.rand_name(
            name='project', prefix=CONF.resource_name_prefix)
        project = self.setup_test_project(
            name=project_name, domain_id=domain_id, parent_id=root_project_id)
        parent_id = project['parent_id']
        self.assertEqual(project_name, project['name'])
        self.assertEqual(root_project_id, parent_id)

    @decorators.idempotent_id('a7eb9416-6f9b-4dbb-b71b-7f73aaef59d5')
    def test_create_is_domain_project(self):
        """Test creating is_domain project"""
        project = self.setup_test_project(domain_id=None, is_domain=True)
        # To delete a domain, we need to disable it first
        self.addCleanup(self.projects_client.update_project, project['id'],
                        enabled=False)

        # Check if the is_domain project is correctly returned by both
        # project and domain APIs
        projects_list = self.projects_client.list_projects(
            params={'is_domain': True})['projects']
        project_ids = [p['id'] for p in projects_list]
        self.assertIn(project['id'], project_ids)

        # The domains API return different attributes for the entity, so we
        # compare the entities IDs
        domains_ids = [d['id'] for d in self.domains_client.list_domains()[
            'domains']]
        self.assertIn(project['id'], domains_ids)

    @decorators.idempotent_id('1f66dc76-50cc-4741-a200-af984509e480')
    def test_project_create_enabled(self):
        """Test creating a project that is enabled"""
        project = self.setup_test_project(enabled=True)
        project_id = project['id']
        self.assertTrue(project['enabled'],
                        'Enable should be True in response')
        body = self.projects_client.show_project(project_id)['project']
        self.assertTrue(body['enabled'], 'Enable should be True in lookup')

    @decorators.idempotent_id('78f96a9c-e0e0-4ee6-a3ba-fbf6dfd03207')
    def test_project_create_not_enabled(self):
        """Test creating a project that is not enabled"""
        project = self.setup_test_project(enabled=False)
        self.assertFalse(project['enabled'],
                         'Enable should be False in response')
        body = self.projects_client.show_project(project['id'])['project']
        self.assertFalse(body['enabled'],
                         'Enable should be False in lookup')

    @decorators.idempotent_id('f608f368-048c-496b-ad63-d286c26dab6b')
    def test_project_update_name(self):
        """Test updating name attribute of a project"""
        p_name1 = data_utils.rand_name(
            name='project', prefix=CONF.resource_name_prefix)
        project = self.setup_test_project(name=p_name1)

        resp1_name = project['name']

        p_name2 = data_utils.rand_name(
            name='project2', prefix=CONF.resource_name_prefix)
        body = self.projects_client.update_project(project['id'],
                                                   name=p_name2)['project']
        resp2_name = body['name']
        self.assertNotEqual(resp1_name, resp2_name)

        body = self.projects_client.show_project(project['id'])['project']
        resp3_name = body['name']

        self.assertNotEqual(resp1_name, resp3_name)
        self.assertEqual(p_name1, resp1_name)
        self.assertEqual(resp2_name, resp3_name)

    @decorators.idempotent_id('f138b715-255e-4a7d-871d-351e1ef2e153')
    def test_project_update_desc(self):
        """Test updating description attribute of a project"""
        p_desc = data_utils.rand_name(
            name='desc', prefix=CONF.resource_name_prefix)
        project = self.setup_test_project(description=p_desc)
        resp1_desc = project['description']

        p_desc2 = data_utils.rand_name(
            name='desc2', prefix=CONF.resource_name_prefix)
        body = self.projects_client.update_project(
            project['id'], description=p_desc2)['project']
        resp2_desc = body['description']
        self.assertNotEqual(resp1_desc, resp2_desc)

        body = self.projects_client.show_project(project['id'])['project']
        resp3_desc = body['description']

        self.assertNotEqual(resp1_desc, resp3_desc)
        self.assertEqual(p_desc, resp1_desc)
        self.assertEqual(resp2_desc, resp3_desc)

    @decorators.idempotent_id('b6b25683-c97f-474d-a595-55d410b68100')
    def test_project_update_enable(self):
        """Test updating the enabled attribute of a project"""
        p_en = False
        project = self.setup_test_project(enabled=p_en)

        resp1_en = project['enabled']

        p_en2 = True
        body = self.projects_client.update_project(project['id'],
                                                   enabled=p_en2)['project']
        resp2_en = body['enabled']
        self.assertNotEqual(resp1_en, resp2_en)

        body = self.projects_client.show_project(project['id'])['project']
        resp3_en = body['enabled']

        self.assertNotEqual(resp1_en, resp3_en)
        self.assertFalse(project['enabled'])
        self.assertEqual(resp2_en, resp3_en)

    @decorators.idempotent_id('59398d4a-5dc5-4f86-9a4c-c26cc804d6c6')
    @testtools.skipIf(CONF.identity_feature_enabled.immutable_user_source,
                      'Skipped because environment has an '
                      'immutable user source and solely '
                      'provides read-only access to users.')
    def test_associate_user_to_project(self):
        """Test associating a user to a project"""
        # Create a Project
        project = self.setup_test_project()

        # Create a User
        u_name = data_utils.rand_name(
            name='user', prefix=CONF.resource_name_prefix)
        u_desc = u_name + 'description'
        u_email = u_name + '@testmail.tm'
        u_password = data_utils.rand_password()
        user = self.users_client.create_user(
            name=u_name, description=u_desc, password=u_password,
            email=u_email, project_id=project['id'])['user']
        # Delete the User at the end of this method
        self.addCleanup(self.users_client.delete_user, user['id'])

        # Get User To validate the user details
        new_user_get = self.users_client.show_user(user['id'])['user']
        # Assert response body of GET
        self.assertEqual(u_name, new_user_get['name'])
        self.assertEqual(u_desc, new_user_get['description'])
        self.assertEqual(project['id'],
                         new_user_get['project_id'])
        self.assertEqual(u_email, new_user_get['email'])

    @decorators.idempotent_id('d1db68b6-aebe-4fa0-b79d-d724d2e21162')
    def test_project_get_equals_list(self):
        """Test the result of getting project equals that of listing"""
        fields = ['parent_id', 'is_domain', 'description', 'links',
                  'name', 'enabled', 'domain_id', 'id', 'tags']

        # Tags must be unique, keystone API will reject duplicates
        tags = ['a', 'c', 'b', 'd']

        # Create a Project, cleanup is handled in the helper
        project = self.setup_test_project(tags=tags)

        # Show and list for the project
        project_get = self.projects_client.show_project(
            project['id'])['project']
        _projects = self.projects_client.list_projects()['projects']
        project_list = next(x for x in _projects if x['id'] == project['id'])

        # Assert the expected fields exist. More fields than expected may
        # be in this list. This is for future proofind as keystone does not
        # and has no plans to support microservices. Any fields in the future
        # that are added to the response of the API should eventually be added
        # to the expected fields. The expected fields must be a subset of
        # the project_get fields (all keys in fields must exist in project_get,
        # but project_get.keys() may have additional fields)
        self.assertTrue(set(fields).issubset(project_get.keys()))

        # Ensure the set of tags is identical and match the expected one
        get_tags = set(project_get.pop("tags"))
        self.assertSetEqual(get_tags, set(project_list.pop("tags")))
        self.assertSetEqual(get_tags, set(tags))

        # Ensure all other fields are identical
        self.assertDictEqual(project_get, project_list)



# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import socket
import struct
import urllib.parse as urlparse

from tempest.api.compute import base
from tempest import config
from tempest.lib import decorators

CONF = config.CONF


class SpiceDirectConsoleTestJSON(base.BaseV2ComputeAdminTest):
    """Test the spice-direct console"""

    create_default_network = True

    min_microversion = '2.99'
    max_microversion = 'latest'

    # SPICE client protocol constants
    magic = b'REDQ'
    major = 2
    minor = 2
    main_channel = 1
    common_caps = 11  # AuthSelection, AuthSpice, MiniHeader
    channel_caps = 9  # SemiSeamlessMigrate, SeamlessMigrate

    @classmethod
    def skip_checks(cls):
        super().skip_checks()
        if not CONF.compute_feature_enabled.spice_console:
            raise cls.skipException('SPICE console feature is disabled.')

    def tearDown(self):
        super().tearDown()
        # NOTE(zhufl): Because server_check_teardown will raise Exception
        # which will prevent other cleanup steps from being executed, so
        # server_check_teardown should be called after super's tearDown.
        self.server_check_teardown()

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.client = cls.servers_client

    @classmethod
    def resource_setup(cls):
        super().resource_setup()
        cls.server = cls.create_test_server(wait_until="ACTIVE")

    @decorators.idempotent_id('80f4460d-1a06-403c-9e93-cf434c70be05')
    def test_spice_direct(self):
        """Test accessing spice-direct console of server"""

        # Request a spice-direct console and validate the result. Any user can
        # do this.
        body = self.servers_client.get_remote_console(
            self.server['id'], console_type='spice-direct', protocol='spice')

        console_url = body['remote_console']['url']
        parts = urlparse.urlparse(console_url)
        qparams = urlparse.parse_qs(parts.query)
        self.assertIn('token', qparams)
        self.assertNotEmpty(qparams['token'])
        self.assertEqual(1, len(qparams['token']))

        self.assertEqual('spice', body['remote_console']['protocol'])
        self.assertEqual('spice-direct', body['remote_console']['type'])

        # For reasons best know to the python developers, the qparams values
        # are lists as documented at
        # https://docs.python.org/3/library/urllib.parse.html
        token = qparams['token'][0]

        # Turn that console token into hypervisor connection details. Only
        # admins can do this because its expected that the request is coming
        # from a proxy and we don't want to expose intimate hypervisor details
        # to all users.
        body = self.admin_servers_client.get_console_auth_token_details(
            token)

        console = body['console']
        self.assertEqual(self.server['id'], console['instance_uuid'])
        self.assertIn('port', console)
        self.assertIn('tls_port', console)
        self.assertIsNone(console['internal_access_path'])

        # Connect to the specified non-TLS port and verify we get back
        # a SPICE protocol greeting
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        sock.connect((console['host'], console['port']))

        # Send a client greeting
        #
        # ---- SpiceLinkMess ----
        # 4s    UINT32 magic value, must be REDQ
        # I     UINT32 major_version, must be 2
        # I     UINT32 minor_version, must be 2
        # I     UINT32 size number of bytes following this field to the end
        #              of this message.
        # I     UINT32 connection_id. In case of a new session (i.e., channel
        #              type is SPICE_CHANNEL_MAIN) this field is set to zero,
        #              and in response the server will allocate session id
        #              and will send it via the SpiceLinkReply message. In
        #              case of all other channel types, this field will be
        #              equal to the allocated session id.
        # B     UINT8  channel_type, we use main
        # B     UINT8  channel_id to connect to
        # I     UINT32 num_common_caps number of common client channel
        #              capabilities words
        # I     UINT32 num_channel_caps number of specific client channel
        #              capabilities words
        # I     UINT32 caps_offset location of the start of the capabilities
        #              vector given by the bytes offset from the “size”
        #              member (i.e., from the address of the “connection_id”
        #              member).
        # ...          capabilities
        sock.sendall(struct.pack(
            '<4sIIIIBBIIIII', self.magic, self.major, self.minor, 42 - 16,
            0, self.main_channel, 0, 1, 1, 18, self.common_caps,
            self.channel_caps))

        # ---- SpiceLinkReply ----
        # 4s     UINT32 magic value, must be equal to SPICE_MAGIC
        # I      UINT32 major_version, must be equal to SPICE_VERSION_MAJOR
        # I      UINT32 minor_version, must be equal to SPICE_VERSION_MINOR
        # I      UINT32 size number of bytes following this field to the end
        #               of this message.
        # I      UINT32 error code
        # ...
        buffered = sock.recv(20)
        self.assertIsNotNone(buffered)
        self.assertEqual(20, len(buffered))

        magic, major, minor, _, error = struct.unpack_from('<4sIIII', buffered)
        self.assertEqual(b'REDQ', magic)
        self.assertEqual(2, major)
        self.assertEqual(2, minor)
        self.assertEqual(0, error)
